import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import dayjs from 'dayjs';
import { computed, defineComponent, inject, ref, watch } from 'vue';
import List from '../../list';
import { useBaseInputComponent } from '../../mixins/base-input-component';
import { pureInputComponentProps } from '../../mixins/pure-input-component';
import { simpleFormComponentProps } from '../../mixins/simple-form-component';
import Calendar from './index';
var MIN_DATE = new Date(2000, 1, 1, 0, 0, 0);
var MAX_DATE = new Date(new Date().getFullYear() + 10, 12, 31, 23, 59, 59);
export default defineComponent({
  name: 'MCalendarItem',
  props: _objectSpread(_objectSpread(_objectSpread({}, simpleFormComponentProps), pureInputComponentProps), {}, {
    /**
     * 标题
     */
    title: {
      type: [String, Object]
    },
    /**
     * 默认值
     */
    defaultDate: {
      type: Date,
      default: function _default() {
        return new Date();
      }
    },
    minDate: {
      type: Date,
      default: function _default() {
        return MIN_DATE;
      }
    },
    maxDate: {
      type: Date,
      default: function _default() {
        return MAX_DATE;
      }
    },
    pickTime: {
      type: Boolean,
      default: false
    },
    type: {
      type: String,
      default: 'range'
    },
    placeholder: {
      type: String
    }
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      attrs = _ref.attrs,
      slots = _ref.slots;
    var form = inject('list', undefined);
    var _useBaseInputComponen = useBaseInputComponent(props, {
        emit: emit,
        attrs: attrs,
        slots: slots
      }, form),
      getDefaultSlot = _useBaseInputComponen.getDefaultSlot,
      inputSlots = _useBaseInputComponen.slots,
      cssStyle = _useBaseInputComponen.cssStyle,
      listeners = _useBaseInputComponen.listeners,
      inputProps = _useBaseInputComponen.props;
    var currentValue = ref([]);
    var open = ref(false);
    watch(function () {
      return props.value;
    }, function (value) {
      if (props.type === 'one') {
        currentValue.value = [value];
      } else if (value) {
        currentValue.value = value;
      }
    }, {
      immediate: true
    });
    var onUpdate = function onUpdate(v) {
      if (v === null || v === undefined) {
        currentValue.value = [];
      } else {
        currentValue.value = v;
      }
    };
    var displayValue = computed(function () {
      var valueStrs = currentValue.value.map(function (it) {
        if (!it) {
          return null;
        }
        if (props.pickTime) {
          return dayjs(it).format('YYYY/MM/DD HH:mm');
        } else {
          return dayjs(it).format('YYYY/MM/DD');
        }
      });
      if (props.type === 'range') {
        if (valueStrs[0] || valueStrs[1]) {
          var _valueStrs$, _valueStrs$2;
          return ((_valueStrs$ = valueStrs[0]) !== null && _valueStrs$ !== void 0 ? _valueStrs$ : '') + ' ~ ' + ((_valueStrs$2 = valueStrs[1]) !== null && _valueStrs$2 !== void 0 ? _valueStrs$2 : '');
        }
        return '';
      } else {
        var _valueStrs$3;
        return (_valueStrs$3 = valueStrs[0]) !== null && _valueStrs$3 !== void 0 ? _valueStrs$3 : '';
      }
    });
    var onClick = function onClick() {
      open.value = true;
    };
    var onConfirm = function onConfirm(value1, value2) {
      if (props.type === 'range') {
        currentValue.value = [value1, value2];
        emit('update:value', [value1, value2]);
      } else {
        currentValue.value = [value1];
        emit('update:value', value1);
      }
    };
    var onClose = function onClose() {
      open.value = false;
    };
    return {
      onClose: onClose,
      inputProps: inputProps,
      displayValue: displayValue,
      onClick: onClick,
      onConfirm: onConfirm,
      listeners: listeners,
      getDefaultSlot: getDefaultSlot,
      cssStyle: cssStyle,
      inputSlots: inputSlots,
      open: open,
      currentValue: currentValue,
      onUpdate: onUpdate
    };
  },
  render: function render() {
    var _this = this;
    var slots = _objectSpread(_objectSpread({}, this.$slots), {}, {
      default: function _default() {
        return [_createVNode(Calendar, _objectSpread(_objectSpread(_objectSpread({}, _this.inputProps), _this.listeners), {}, {
          "onUpdate:value": _this.onUpdate,
          "value": _this.currentValue,
          'open': _this.open,
          "onUpdate:open": function onUpdateOpen($event) {
            return _this.open = $event;
          },
          "onClose": _this.onClose,
          "onConfirm": _this.onConfirm,
          "defaultValue": _this.currentValue,
          "style": _this.cssStyle
        }), _objectSpread({
          default: function _default() {
            return [_this.getDefaultSlot()];
          }
        }, _this.inputSlots)), _createVNode("span", null, [_this.title])];
      },
      control: function control() {
        return _createVNode("span", null, [_this.displayValue || _this.placeholder]);
      }
    });
    return _createVNode(List.Item, {
      "text": !!this.displayValue,
      "class": 'am-calendar-item',
      "required": this.required,
      "touchFeedback": true,
      "disabled": this.disabled,
      "error": this.error,
      "errorMessage": this.errorMessage,
      "errorDisplayType": this.errorDisplayType,
      "arrow": "horizontal",
      "title": this.title,
      "onClick": this.onClick
    }, slots);
  }
});