import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { createVNode as _createVNode } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import classnames from 'classnames';
import { defineComponent, ref } from 'vue';
import CarouselBase from './base';
var DotDecorator = defineComponent({
  name: 'DotDecorator',
  inheritAttrs: false,
  props: {
    slideCount: {
      type: Number
    },
    slidesToScroll: {
      type: Number
    },
    currentSlide: {
      type: Number
    },
    prefixCls: {
      type: String,
      default: 'am-carousel'
    },
    dotActiveStyle: {
      type: Object
    },
    dotStyle: {
      type: Object
    }
  },
  render: function render() {
    var _this = this;
    var arr = [];
    for (var i = 0; i < this.slideCount; i += this.slidesToScroll) {
      arr.push(i);
    }
    var dotDom = arr.map(function (index) {
      var dotCls = classnames("".concat(_this.prefixCls, "-wrap-dot"), _defineProperty({}, "".concat(_this.prefixCls, "-wrap-dot-active"), index === _this.currentSlide));
      var currentDotStyle = index === _this.currentSlide ? _this.dotActiveStyle : _this.dotStyle;
      return _createVNode("div", {
        "class": dotCls,
        "key": index
      }, [_createVNode("span", {
        "style": currentDotStyle
      }, null)]);
    });
    return _createVNode("div", {
      "class": "".concat(this.prefixCls, "-wrap")
    }, [dotDom]);
  }
});
var Carousel = defineComponent({
  name: 'MCarousel',
  props: {
    prefixCls: {
      type: String,
      default: 'am-carousel'
    },
    beforeChange: {},
    afterChange: {
      type: Function
    },
    swipeSpeed: {},
    easing: {},
    dotStyle: {
      default: function _default() {
        return {};
      }
    },
    dotActiveStyle: {
      default: function _default() {
        return {};
      }
    },
    frameOverflow: {
      type: String
    },
    cellAlign: {
      type: String,
      default: 'center'
    },
    cellSpacing: {
      type: Number
    },
    slideWidth: {
      type: [String, Number]
    },
    dots: {
      type: Boolean,
      default: true
    },
    vertical: {
      type: Boolean
    },
    autoplay: {
      type: Boolean,
      default: false
    },
    autoplayInterval: {
      type: Number
    },
    infinite: {
      type: Boolean,
      default: false
    },
    initialSlideWidth: {
      type: Number
    }
  },
  setup: function setup(props) {
    var selectedIndex = ref(0);
    var onChange = function onChange(index) {
      selectedIndex.value = index;
      if (props.afterChange) {
        props.afterChange(index);
      }
    };
    return {
      onChange: onChange,
      selectedIndex: selectedIndex
    };
  },
  render: function render() {
    var _this2 = this;
    var infinite = this.infinite,
      selectedIndex = this.selectedIndex,
      beforeChange = this.beforeChange,
      dots = this.dots,
      prefixCls = this.prefixCls,
      vertical = this.vertical;
    var newProps = _objectSpread(_objectSpread({}, this.$props), {}, {
      wrapAround: infinite,
      slideIndex: selectedIndex,
      beforeSlide: beforeChange
    });
    var Decorators = [];
    if (dots) {
      Decorators = [{
        component: DotDecorator,
        position: 'BottomCenter'
      }];
    }
    var wrapCls = classnames(prefixCls, _defineProperty({}, "".concat(prefixCls, "-vertical"), vertical), this.$attrs.class);
    return _createVNode(CarouselBase, _objectSpread(_objectSpread({}, _objectSpread(_objectSpread({}, newProps), {}, {
      decorators: Decorators,
      afterSlide: this.onChange
    })), {}, {
      "class": wrapCls
    }), {
      default: function _default() {
        return [_this2.$slots.default()];
      }
    });
  }
});
export default Carousel;