import { createVNode as _createVNode } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _createForOfIteratorHelper from "@babel/runtime/helpers/esm/createForOfIteratorHelper";
import { computed, defineComponent, nextTick, ref, watch } from 'vue';
import { usePureInput } from '../mixins/pure-input-component';
import RadioList from '../radio/src/radio-list';
import CheckboxList from '../checkbox/src/checkbox-list';
import Tabs from '../tabs';
import { flatten } from 'lodash';
var classPrefix = "adm-cascader-view";
export var CascaderView = defineComponent({
  name: 'MCascaderView',
  props: {
    options: Array,
    value: Array,
    disabled: Boolean,
    animated: Boolean,
    loadData: Function,
    defaultValue: {
      type: Array,
      default: function _default() {
        return [];
      }
    },
    filterOption: Function,
    onChange: Function,
    placeholder: {
      type: String,
      default: '请选择'
    },
    multiple: Boolean
  },
  emits: ['update:value', 'change'],
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      attrs = _ref.attrs,
      slots = _ref.slots;
    var _usePureInput = usePureInput(props, {
        emit: emit,
        attrs: attrs,
        slots: slots
      }, {
        defaultValue: props.defaultValue,
        propName: 'value'
      }),
      stateValue = _usePureInput.stateValue;
    var tabActiveKey = ref(0);
    var levels = computed(function () {
      var ret = [];
      var currentOptions = props.options;
      var reachedEnd = false;
      var _iterator = _createForOfIteratorHelper(stateValue.value || []),
        _step;
      try {
        var _loop = function _loop() {
          var _target, _target$children;
          var v = _step.value;
          var target;
          if (Array.isArray(v)) {
            var selected = currentOptions.filter(function (it) {
              return v.includes(it.value);
            });
            if (selected.length) {
              target = {
                value: selected.map(function (it) {
                  return it.value;
                }),
                label: selected.length + '个选项',
                children: flatten(selected.map(function (it) {
                  return it.children || [];
                }))
              };
            }
          } else {
            target = currentOptions.find(function (option) {
              return option.value === v;
            });
          }
          ret.push({
            selected: target,
            options: currentOptions
          });
          if (!((_target = target) !== null && _target !== void 0 && (_target$children = _target.children) !== null && _target$children !== void 0 && _target$children.length)) {
            reachedEnd = true;
            return 1; // break
          }
          currentOptions = target.children;
        };
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          if (_loop()) break;
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
      if (!reachedEnd) {
        ret.push({
          selected: undefined,
          options: currentOptions
        });
      }
      return ret;
    });
    watch(function () {
      return stateValue.value;
    }, function () {
      tabActiveKey.value = levels.value.length - 1;
    });
    var onItemSelect = function onItemSelect(selectValue, depth) {
      if (stateValue.value[depth] === selectValue) {
        stateValue.value = stateValue.value.slice(0, depth);
      } else {
        var next = stateValue.value.slice(0, depth);
        if (selectValue !== undefined) {
          next[depth] = selectValue;
        }
        stateValue.value = next;
      }
    };
    watch(function () {
      return levels.value;
    }, function () {
      var selected = levels.value[levels.value.length - 1].selected;
      var options = levels.value.map(function (it) {
        return it.selected;
      });
      if (selected && !selected.isLeaf && !selected.children && props.loadData) {
        selected.loading = true;
        props.loadData(options).then(function () {
          nextTick(function () {
            tabActiveKey.value = levels.value.length - 1;
          });
        }).catch(function (e) {
          console.error(e);
        }).finally(function () {
          selected.loading = false;
        });
      }
    });
    return {
      levels: levels,
      onItemSelect: onItemSelect,
      setTabActiveKey: function setTabActiveKey(key) {
        tabActiveKey.value = key;
      },
      stateValue: stateValue,
      tabActiveKey: tabActiveKey
    };
  },
  render: function render() {
    var _this = this;
    return _createVNode("div", {
      "class": classPrefix
    }, [_createVNode(Tabs, {
      "value": this.tabActiveKey,
      "swipeable": false,
      "animated": this.animated,
      "onChange": function onChange(_, index) {
        _this.setTabActiveKey(index);
      },
      "class": "".concat(classPrefix, "-tabs")
    }, {
      default: function _default() {
        return [_this.levels.map(function (level, index) {
          var selected = level.selected;
          var radioProps = _defineProperty(_defineProperty(_defineProperty(_defineProperty({
            value: _this.stateValue[index]
          }, 'onUpdate:value', function onUpdateValue(v) {
            _this.onItemSelect(v, index);
          }), "iconType", 'check'), "options", _this.filterOption ? level.options.filter(_this.filterOption) : level.options), "class", "".concat(classPrefix, "-content"));
          var hasChildren = level.options.some(function (it) {
            var _it$children;
            return (_it$children = it.children) === null || _it$children === void 0 ? void 0 : _it$children.length;
          });
          return _createVNode(Tabs.Tab, {
            "key": "tab_".concat(index),
            "title": _createVNode("div", {
              "class": "".concat(classPrefix, "-header-title")
            }, [selected ? selected.label : _this.placeholder]),
            "forceRender": true
          }, {
            default: function _default() {
              return [_this.multiple && index === _this.levels.length - 1 && !hasChildren ? _createVNode(CheckboxList, radioProps, null) : _createVNode(RadioList, radioProps, null)];
            }
          });
        })];
      }
    })]);
  }
});