import { createTextVNode as _createTextVNode, createVNode as _createVNode } from "vue";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { computed, defineComponent, inject, ref, watch } from "vue";
import List from '../../list';
import CascaderView from "../../cascader-view";
import Popup from "../../popup";
import { optionsBasedComponentProps, useOptionsBaseComponent } from "../../mixins/options-based-component";
import classNames from "classnames";
import { getOptionProperty } from "../../utils/option";
import Search from '../../search-bar';
export default defineComponent({
  name: 'MCascader',
  props: _objectSpread(_objectSpread({}, optionsBasedComponentProps), {}, {
    title: [String, Object],
    disabled: Boolean,
    options: Array,
    popupHeight: [Number, String],
    value: Array,
    animated: Boolean,
    loadData: Function,
    clearable: Boolean,
    defaultValue: {
      type: Array,
      default: function _default() {
        return [];
      }
    },
    searchable: Boolean,
    onChange: Function,
    placeholder: {
      type: String,
      default: '请选择'
    },
    multiple: Boolean
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        slots: slots,
        attrs: attrs
      }, form),
      getOptions = _useOptionsBaseCompon.getOptions,
      filteredValues = _useOptionsBaseCompon.filteredValues,
      searchKeyword = _useOptionsBaseCompon.searchKeyword,
      isReadonly = _useOptionsBaseCompon.isReadonly,
      isDisabled = _useOptionsBaseCompon.isDisabled;
    var stateValue = ref();
    watch(function () {
      return props.value;
    }, function () {
      if (props.value) {
        stateValue.value = _toConsumableArray(props.value);
      } else {
        stateValue.value = [];
      }
    }, {
      immediate: true
    });
    var optionText = computed(function () {
      if (props.value) {
        var options = props.options;
        var selected = [];
        props.value.forEach(function (v) {
          var option = options.find(function (it) {
            return getOptionProperty(it, props.valueProperty) === v;
          });
          selected.push(option);
          options = (option === null || option === void 0 ? void 0 : option.children) || [];
        });
        return selected.map(function (it) {
          return getOptionProperty(it, props.labelProperty);
        }).join(' / ');
      } else {
        return undefined;
      }
    });
    var popupOpen = ref(false);
    var onClick = function onClick() {
      if (!isDisabled.value && !isReadonly.value) {
        var _props$value;
        stateValue.value = _toConsumableArray((_props$value = props.value) !== null && _props$value !== void 0 ? _props$value : []);
        popupOpen.value = true;
      }
    };
    var confirm = function confirm() {
      popupOpen.value = false;
      emit('update:value', stateValue.value);
    };
    var close = function close() {
      popupOpen.value = false;
    };
    var onClear = function onClear() {
      stateValue.value = [];
      confirm();
    };
    return {
      optionText: optionText,
      searchKeyword: searchKeyword,
      popupOpen: popupOpen,
      confirm: confirm,
      stateValue: stateValue,
      filteredValues: filteredValues,
      close: close,
      isDisabled: isDisabled,
      isReadonly: isReadonly,
      onClick: onClick,
      onClear: onClear
    };
  },
  render: function render() {
    var _this = this;
    var cancelButton = _createVNode("div", {
      "onClick": this.onClear,
      "class": "am-popup-item am-popup-header-left"
    }, [_createTextVNode("\u6E05\u9664")]);
    var placeholder = _createVNode("span", {
      "class": 'am-list-item-placeholder'
    }, [this.placeholder]);
    return [_createVNode(List.Item, {
      "disabled": this.disabled,
      "onClick": this.onClick,
      "error": this.error,
      "errorMessage": this.errorMessage,
      "arrow": 'horizontal',
      "errorDisplayType": this.errorDisplayType,
      "style": this.$attrs.style,
      "class": classNames(this.$attrs.class, {
        'am-checkbox-popup-list': true,
        'am-checkbox-popup-list-focus': this.isDisabled ? false : this.popupOpen
      }),
      "touchFeedback": !this.isReadonly && !this.isDisabled,
      "required": this.required,
      "text": !!this.optionText
    }, {
      default: function _default() {
        return _this.title;
      },
      control: function control() {
        return _createVNode("div", null, [_this.optionText, (!_this.stateValue || _this.stateValue.length === 0) && _this.placeholder && placeholder]);
      }
    }), _createVNode(Popup, {
      "open": this.disabled ? false : this.popupOpen,
      "showCancel": this.clearable,
      "height": this.popupHeight,
      "cancelButton": cancelButton,
      "title": this.title,
      "onOk": this.confirm,
      "onCancel": this.close
    }, {
      default: function _default() {
        return [_this.searchable && _createVNode(Search, {
          'value': _this.searchKeyword,
          "onUpdate:value": function onUpdateValue($event) {
            return _this.searchKeyword = $event;
          }
        }, null), _createVNode(CascaderView, {
          "options": _this.options,
          "animated": _this.animated,
          "defaultValue": _this.defaultValue,
          "filterOption": function filterOption(option) {
            if (_this.searchKeyword) {
              return _this.filteredValues.includes(getOptionProperty(option, _this.valueProperty));
            }
            return true;
          },
          "loadData": _this.loadData,
          'value': _this.stateValue,
          "onUpdate:value": function onUpdateValue($event) {
            return _this.stateValue = $event;
          },
          "multiple": _this.multiple,
          "disabled": _this.disabled
        }, null)];
      }
    })];
  }
});