import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent, getCurrentInstance, inject, onMounted } from 'vue';
import List from '../../list';
import { optionsBasedComponentProps, useOptionsBaseComponent } from '../../mixins/options-based-component';
import SearchBar from '../../search-bar/src';
import CheckboxItem from './checkbox-item';
export default defineComponent({
  name: 'MCheckboxList',
  props: _objectSpread(_objectSpread({}, optionsBasedComponentProps), {}, {
    value: {
      type: Array
    },
    title: {
      type: String
    },
    maxHeightPercentage: {
      type: Number
    },
    onChange: Function
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        attrs: attrs,
        slots: slots
      }, form, {
        defaultValue: [],
        propName: 'value'
      }),
      getOptions = _useOptionsBaseCompon.getOptions,
      setStateValue = _useOptionsBaseCompon.setStateValue,
      isReadonly = _useOptionsBaseCompon.isReadonly,
      stateValue = _useOptionsBaseCompon.stateValue,
      searchKeyword = _useOptionsBaseCompon.searchKeyword,
      isDisabled = _useOptionsBaseCompon.isDisabled;
    var renderOptions = function renderOptions() {
      var options = getOptions();
      return options.map(function (option, index) {
        var optionProps = {};
        return _createVNode(CheckboxItem, _objectSpread(_objectSpread({}, optionProps), {}, {
          "value": stateValue.value.includes(option.value),
          "key": option.value,
          "disabled": option.disabled || isDisabled.value,
          "readOnly": isReadonly.value,
          "onChange": function onChange(checkState) {
            _onChange(checkState, option.value);
          }
        }), {
          default: function _default() {
            return [option.label];
          }
        });
      });
    };
    var _onChange = function _onChange(checkState, value) {
      if (checkState) {
        if (!stateValue.value.includes(value)) {
          setStateValue([].concat(_toConsumableArray(stateValue.value), [value]));
        }
      } else {
        setStateValue(stateValue.value.filter(function (it) {
          return it !== value;
        }));
      }
    };
    var renderSearch = function renderSearch() {
      return props.searchable ? _createVNode(SearchBar, _objectSpread({
        "value": searchKeyword.value
      }, _defineProperty({}, 'onUpdate:value', function onUpdateValue(v) {
        searchKeyword.value = v;
      })), null) : null;
    };
    var instance = getCurrentInstance();
    onMounted(function () {
      if (props.maxHeightPercentage) {
        var windowHeight = document.body.clientHeight;
        var maxHeight = props.maxHeightPercentage;
        if (instance.vnode.el.clientHeight > windowHeight * maxHeight) {
          instance.vnode.el.style.height = windowHeight * maxHeight + 'px';
        }
      }
    });
    return {
      renderOptions: renderOptions,
      renderSearch: renderSearch
    };
  },
  render: function render() {
    var _this = this;
    return _createVNode(List, {
      "required": this.required,
      "errorMessage": this.errorMessage,
      "title": this.title
    }, {
      default: function _default() {
        return [_this.renderSearch(), _this.renderOptions()];
      }
    });
  }
});