import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createTextVNode as _createTextVNode, createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import classNames from 'classnames';
import omit from 'omit.js';
import { computed, defineComponent, inject, ref, watch } from 'vue';
import List from '../../list';
import { optionsBasedComponentProps, useOptionsBaseComponent } from '../../mixins/options-based-component';
import Popup from '../../popup';
import CheckboxList from './checkbox-list';
export default defineComponent({
  name: 'MCheckboxPopupList',
  props: _objectSpread(_objectSpread({}, optionsBasedComponentProps), {}, {
    title: {
      type: [String, Object]
    },
    placeholder: {
      type: String,
      default: '请选择'
    },
    clearable: {
      type: Boolean,
      default: false
    },
    separator: {
      type: String,
      default: '、'
    },
    open: {
      type: Boolean,
      default: false
    }
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        slots: slots,
        attrs: attrs
      }, form),
      getOptions = _useOptionsBaseCompon.getOptions,
      searchKeyword = _useOptionsBaseCompon.searchKeyword,
      isReadonly = _useOptionsBaseCompon.isReadonly,
      stateValue = _useOptionsBaseCompon.stateValue,
      isDisabled = _useOptionsBaseCompon.isDisabled;
    var popupOpen = ref(props.open);
    watch(function () {
      return props.open;
    }, function (open) {
      popupOpen.value = open;
    });
    watch(function () {
      return popupOpen.value;
    }, function (open) {
      emit('update:open', open);
    });
    var optionText = computed(function () {
      var options = getOptions();
      // @ts-ignore
      var value = stateValue.value;
      var array = [];
      if (value) {
        value.forEach(function (v, index) {
          var option = options.find(function (it) {
            return it.value === v;
          });
          if (option) {
            array.push(option.label);
          } else {
            array.push(v);
          }
          if (index < value.length - 1) {
            array.push(props.separator);
          }
        });
      }
      return array;
    });
    var onChange = function onChange(value) {
      stateValue.value = value;
      emit('update:value', stateValue.value);
      emit('change', stateValue.value);
    };
    var onClick = function onClick() {
      if (!isDisabled.value && !isReadonly.value) {
        popupOpen.value = true;
      }
    };
    var onClear = function onClear() {
      emit('clear');
      emit('update:value', []);
      closePopup();
    };
    var closePopup = function closePopup() {
      popupOpen.value = false;
    };
    return {
      onChange: onChange,
      stateValue: stateValue,
      getOptions: getOptions,
      onClick: onClick,
      isDisabled: isDisabled,
      isReadonly: isReadonly,
      closePopup: closePopup,
      onClear: onClear,
      popupOpen: popupOpen,
      optionText: optionText
    };
  },
  render: function render() {
    var _this = this;
    var listProps = _objectSpread(_objectSpread(_objectSpread({}, this.$attrs), this.$props), {}, {
      options: this.getOptions()
    });
    var stateValue = this.stateValue,
      placeholder = this.placeholder;
    listProps.title = undefined;
    var cancelButton = _createVNode("div", {
      "onClick": this.onClear,
      "class": "am-popup-item am-popup-header-left"
    }, [_createTextVNode("\u6E05\u9664")]);
    var slots = {
      control: function control() {
        return _createVNode("span", {
          "class": _defineProperty({}, "am-list-item-placeholder", !stateValue || stateValue.length === 0)
        }, [stateValue && stateValue.length ? _this.optionText : placeholder]);
      },
      default: function _default() {
        return _createVNode("span", null, [_this.title]);
      }
    };
    return [_createVNode(List.Item, {
      "onClick": this.onClick,
      "error": this.error,
      "errorMessage": this.errorMessage,
      "arrow": 'horizontal',
      "errorDisplayType": this.errorDisplayType,
      "style": this.$attrs.style,
      "class": classNames(this.$attrs.class, {
        'am-checkbox-popup-list': true,
        'am-checkbox-popup-list-focus': this.isDisabled ? false : this.popupOpen
      }),
      "touchFeedback": !this.isReadonly && !this.isDisabled,
      "required": this.required,
      "text": !!this.optionText,
      "disabled": this.isDisabled
    }, slots), _createVNode(Popup, {
      "open": this.isDisabled ? false : this.popupOpen,
      "showCancel": this.clearable,
      "cancelButton": cancelButton,
      "title": this.title,
      "onOk": this.closePopup,
      "onCancel": this.closePopup
    }, {
      default: function _default() {
        return [_createVNode(CheckboxList, _objectSpread(_objectSpread({}, omit(listProps, ['errorMessage', 'error', 'errorDisplayType'])), {}, {
          "maxHeightPercentage": 0.7,
          "onChange": _this.onChange
        }), null)];
      }
    })];
  }
});