import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent, provide, ref, watch, createVNode as _createVNode } from 'vue';
import { getComponentLocale } from '../../utils/getLocale';
import { setProps } from '../../utils/vnode';
import RCDatePicker from '../../vmc-date-picker/date-picker';
import PopupDatePicker from '../../vmc-date-picker/popup';
import locale from './locale/zh_CN';
import defaultLocale from './locale/zh_CN';
import { formatFn } from './utils';
export var getDatePicker = function getDatePicker(isView, name) {
  return defineComponent({
    install: null,
    Item: null,
    name: name,
    props: {
      disabled: {
        type: Boolean
      },
      cancelText: [String, Object],
      editable: {
        type: Boolean,
        default: true
      },
      extra: {
        type: String
      },
      format: {
        type: [String, Function]
      },
      locale: {
        type: Object,
        default: function _default() {
          return locale;
        }
      },
      maxDate: {
        type: Date
      },
      disabledDate: {
        type: Function
      },
      minDate: {
        type: Date
      },
      minuteStep: {
        type: Number,
        default: 1
      },
      mode: {
        type: String,
        default: 'datetime'
      },
      okText: [String, Object],
      title: [String, Object],
      value: {
        type: [Object, Number, String]
      },
      open: {
        type: Boolean,
        default: false
      },
      placeholder: {
        type: String,
        default: ''
      },
      prefixCls: {
        type: String,
        default: 'am-picker'
      },
      use12Hours: {
        type: Boolean,
        default: false
      },
      pickerPrefixCls: {
        type: String,
        default: 'am-picker-col'
      },
      popupPrefixCls: {
        type: String,
        default: 'am-picker-popup'
      }
    },
    emits: ['change', 'ok', 'update:value', 'update:open'],
    setup: function setup(props, _ref) {
      var emit = _ref.emit;
      var getDate = function getDate() {
        if (typeof props.value === 'number') {
          return new Date(props.value);
        } else if (typeof props.value === 'string') {
          return new Date(props.value);
        } else {
          return props.value || new Date();
        }
      };
      var scrollValue = ref(getDate());
      var localOpen = ref(props.open);
      watch(function () {
        return props.open;
      }, function (open) {
        localOpen.value = open;
      });
      var onOk = function onOk() {
        var value = props.value;
        if (scrollValue.value !== undefined) {
          value = scrollValue.value;
        }
        emit('update:value', value);
        emit('change', value);
        emit('ok', value);
      };
      var onOpenChange = function onOpenChange(open) {
        localOpen.value = open;
        emit('update:open', open);
      };
      var fixOnOk = function fixOnOk(picker) {
        if (picker) {
          picker.onOk = onOk;
        }
      };
      var onChange = function onChange(v) {
        scrollValue.value = v;
        if (isView) {
          emit('update:value', v);
          emit('change', v);
        }
      };
      provide('store', {
        onOk: null
      });
      return {
        scrollValue: scrollValue,
        onOk: onOk,
        onOpenChange: onOpenChange,
        fixOnOk: fixOnOk,
        onChange: onChange,
        getDate: getDate,
        localOpen: localOpen
      };
    },
    render: function render() {
      var _this$$slots$default,
        _this$$slots,
        _this$$slots$default$,
        _this = this;
      var _this$$props = this.$props,
        value = _this$$props.value,
        popupPrefixCls = _this$$props.popupPrefixCls;
      var locale = getComponentLocale(this.$props, null, 'DatePicker', function () {
        return defaultLocale;
      });
      var okText = locale.okText,
        cancelText = locale.cancelText,
        extra = locale.extra,
        DatePickerLocale = locale.DatePickerLocale;

      /**
       * 注意:
       * 受控 表示 通过设置 value 属性、组件的最终状态跟 value 设置值一致。
       * 默认不设置 value 或 只设置 defaultValue 表示非受控。
       *
       * DatePickerView 对外通过 value “只支持 受控” 模式（可以使用 defaultDate 支持 非受控 模式，但不对外）
       * PickerView 对外通过 value “只支持 受控” 模式
       *
       * DatePicker / Picker 对外只有 value 属性 (没有 defaultValue)，
       * 其中 List 展示部分 “只支持 受控” 模式，
       * 弹出的 选择器部分 会随外部 value 改变而变、同时能自由滚动
       * （即不会因为传入的 value 不变而不能滚动 (不像原生 input 的受控行为)）
       *
       */
      var datePickerProps = {
        minuteStep: this.minuteStep,
        locale: DatePickerLocale,
        minDate: this.minDate,
        maxDate: this.maxDate,
        mode: this.mode,
        pickerPrefixCls: this.pickerPrefixCls,
        prefixCls: this.prefixCls,
        disabledDate: this.disabledDate,
        value: this.getDate(),
        use12Hours: this.use12Hours,
        onChange: this.onChange
      };
      var datePicker = _createVNode(RCDatePicker, datePickerProps, null);
      if (isView) {
        return datePicker;
      }
      var textValue = value ? formatFn(value, this.format, this.mode) : null;
      var childExtra = textValue ? textValue : this.extra || _createVNode("span", {
        "class": 'am-list-item-placeholder'
      }, [this.placeholder || extra]);
      var open = this.disabled || !this.editable ? false : this.localOpen;
      return _createVNode(PopupDatePicker, _objectSpread(_objectSpread({
        "onOpenChange": this.onOpenChange,
        "datePicker": datePicker
      }, this.$props), {}, {
        "title": this.title,
        "disabled": this.disabled,
        "editable": this.editable,
        "open": open,
        "prefixCls": popupPrefixCls,
        "date": this.getDate(),
        "cancelText": this.cancelText || cancelText,
        "okText": this.okText || okText,
        "ref": this.fixOnOk
      }), {
        default: function _default() {
          return [(_this$$slots$default = (_this$$slots = _this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : (_this$$slots$default$ = _this$$slots$default.call(_this$$slots)) === null || _this$$slots$default$ === void 0 ? void 0 : _this$$slots$default$.map(function (it) {
            setProps(it, {
              touchFeedback: true,
              onClick: function onClick() {
                _this.onOpenChange(true);
              },
              text: !!textValue,
              control: childExtra,
              arrow: 'horizontal'
            });
            return it;
          })];
        }
      });
    }
  });
};
export default getDatePicker(false, 'MDatePicker');