import { defineComponent, onBeforeUnmount, onMounted, ref, Fragment as _Fragment, createTextVNode as _createTextVNode, createVNode as _createVNode } from 'vue';
import Loading from '../loading';
import { getScrollParent } from '../utils/get-scroll-parent';
import useLockFn from '../utils/use-lock-fn';
import useMemoizedFn from '../utils/use-memoized-fn';
function isWindow(element) {
  return element === window;
}
var classPrefix = "adm-infinite-scroll";
var InfiniteScrollContent = function InfiniteScrollContent(_ref) {
  var hasMore = _ref.hasMore;
  return _createVNode(_Fragment, null, [hasMore ? _createVNode(_Fragment, null, [_createVNode("span", null, [_createTextVNode("\u52A0\u8F7D\u4E2D")]), _createVNode(Loading, null, null)]) : _createVNode("span", null, [_createTextVNode("\u6CA1\u6709\u66F4\u591A\u4E86")])]);
};
export var InfiniteScroll = defineComponent({
  name: 'MInfiniteScroll',
  props: {
    loadMore: {
      type: Function
    },
    hasMore: Boolean,
    threshold: {
      type: Number,
      default: 250
    }
  },
  setup: function setup(props) {
    var doLoadMore = useLockFn(function () {
      return props.loadMore();
    });
    var elementRef = ref(null);
    var flag = ref(2);
    var checkTimeoutRef = ref();
    var check = useMemoizedFn(function () {
      window.clearTimeout(checkTimeoutRef.value);
      checkTimeoutRef.value = window.setTimeout(function () {
        if (!props.hasMore) {
          return;
        }
        var element = elementRef.value;
        if (!element) {
          return;
        }
        if (!element.offsetParent) {
          return;
        }
        var parent = getScrollParent(element);
        if (!parent) {
          return;
        }
        var rect = element.getBoundingClientRect();
        var elementTop = rect.top;
        var current = isWindow(parent) ? window.innerHeight : parent.getBoundingClientRect().bottom;
        if (current >= elementTop - props.threshold) {
          var origin = flag.value;
          flag.value = 1;
          if (origin === 2) {
            doLoadMore();
          }
        } else {
          flag.value = 2;
        }
      });
    });

    // 确保在内容不足时会自动触发加载事件
    check();
    var parent = ref();
    var onScroll = function onScroll() {
      check();
    };
    onMounted(function () {
      var element = elementRef.value;
      if (!element) {
        return;
      }
      parent.value = getScrollParent(element);
      if (!parent) {
        return;
      }
      parent.value.addEventListener('scroll', onScroll);
    });
    onBeforeUnmount(function () {
      parent.value.removeEventListener('scroll', onScroll);
    });
    return {
      elementRef: elementRef
    };
  },
  render: function render() {
    var _this$$slots$default, _this$$slots;
    return _createVNode("div", {
      "class": classPrefix,
      "ref": "elementRef"
    }, [(_this$$slots$default = (_this$$slots = this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : _this$$slots$default.call(_this$$slots), !this.$slots.default && _createVNode(InfiniteScrollContent, {
      "hasMore": this.hasMore
    }, null)]);
  }
});