import { createVNode as _createVNode } from "vue";
import classnames from 'classnames';
import { defineComponent, onBeforeUnmount, ref, Teleport } from 'vue';
import CustomKeyboard from './custom-keyboard';
export default defineComponent({
  name: 'MNumberInput',
  props: {
    onConfirm: Function,
    onChange: Function,
    onFocus: Function,
    onBlur: Function,
    placeholder: {
      type: String,
      default: ''
    },
    disabled: {
      type: Boolean,
      default: false
    },
    editable: {
      type: Boolean,
      default: true
    },
    moneyKeyboardAlign: {},
    moneyKeyboardWrapProps: {},
    moneyKeyboardHeader: {},
    value: {
      type: [String, Number]
    },
    prefixCls: {
      type: String,
      default: 'am-input'
    },
    keyboardPrefixCls: {
      type: String,
      default: 'am-number-keyboard'
    },
    confirmLabel: {},
    backspaceLabel: {
      type: String
    },
    cancelKeyboardLabel: {
      type: String
    },
    maxLength: {
      type: Number
    },
    type: {}
  },
  watch: {
    value: function value(_value) {
      this.currentValue = _value;
    },
    focus: function focus(_focus) {
      if (_focus) {
        this.onInputFocus();
      }
    }
  },
  setup: function setup(props, _ref) {
    var _props$value;
    var emit = _ref.emit;
    var container = ref(null);
    var focus = ref(false);
    var currentValue = ref(null);
    var onChange = function onChange(value) {
      if (props.value === undefined) {
        currentValue.value = value.target.value;
      }
      emit('change', value);
    };
    var onConfirm = function onConfirm(value) {
      emit('confirm', value);
    };
    var addBlurListener = function addBlurListener() {
      document.addEventListener('click', doBlur, false);
    };
    var removeBlurListener = function removeBlurListener() {
      document.removeEventListener('click', doBlur, false);
    };
    var getComponent = function getComponent() {
      var confirmLabel = props.confirmLabel,
        backspaceLabel = props.backspaceLabel,
        cancelKeyboardLabel = props.cancelKeyboardLabel,
        keyboardPrefixCls = props.keyboardPrefixCls,
        moneyKeyboardWrapProps = props.moneyKeyboardWrapProps,
        moneyKeyboardHeader = props.moneyKeyboardHeader;
      return _createVNode(CustomKeyboard, {
        "open": focus.value,
        "onClick": onKeyboardClick,
        "prefixCls": keyboardPrefixCls,
        "confirmLabel": confirmLabel,
        "backspaceLabel": backspaceLabel,
        "cancelKeyboardLabel": cancelKeyboardLabel,
        "wrapProps": moneyKeyboardWrapProps,
        "header": moneyKeyboardHeader
      }, null);
    };
    var getContainer = function getContainer() {
      var keyboardPrefixCls = props.keyboardPrefixCls;
      if (!container.value) {
        var div = document.createElement('div');
        div.setAttribute('id', "".concat(keyboardPrefixCls, "-container-").concat(new Date().getTime()));
        document.body.appendChild(div);
        container.value = div;
      }
      return container.value;
    };
    var doBlur = function doBlur() {
      onInputBlur(props.value.toString());
    };
    var onInputBlur = function onInputBlur(value) {
      if (focus.value) {
        focus.value = false;
        emit('blur', value);
      }
    };
    var onInputFocus = function onInputFocus() {
      emit('focus', props.value);
      focus.value = true;
    };
    var onKeyboardClick = function onKeyboardClick(keyboardItemValue) {
      var maxLength = props.maxLength,
        value = props.value;
      // tslint:disable-next-line:no-this-assignment

      var valueAfterChange;
      // 删除键
      if (keyboardItemValue === 'delete') {
        valueAfterChange = value.toString().substring(0, value.toString().length - 1);
        emit('change', {
          target: {
            value: valueAfterChange
          }
        });
        // 确认键
      } else if (keyboardItemValue === 'confirm') {
        valueAfterChange = value;
        onChange({
          target: {
            value: valueAfterChange
          }
        });
        onInputBlur(value.toString());
        onConfirm(value);
        // 收起键
      } else if (keyboardItemValue === 'hide') {
        valueAfterChange = value;
        onInputBlur(valueAfterChange);
      } else if (maxLength !== undefined && +maxLength >= 0 && (value + keyboardItemValue).length > maxLength) {
        valueAfterChange = (value + keyboardItemValue).substring(0, maxLength);
        onChange({
          target: {
            value: valueAfterChange
          }
        });
      } else {
        valueAfterChange = value + keyboardItemValue;
        onChange({
          target: {
            value: valueAfterChange
          }
        });
      }
    };
    var onFakeInputClick = function onFakeInputClick() {
      focusFunc();
    };
    var focusFunc = function focusFunc() {
      // this focus may invocked by users page button click, so this click may trigger blurEventListener at the same time
      removeBlurListener();
      if (!focus.value) {
        onInputFocus();
      }
      setTimeout(function () {
        addBlurListener();
      }, 50);
    };
    var renderPortal = function renderPortal() {
      var props = {
        disabled: false,
        to: getContainer()
      };
      if (props.to) {
        return _createVNode(Teleport, props, {
          default: function _default() {
            return [getComponent()];
          }
        });
      }
    };
    currentValue.value = ((_props$value = props.value) === null || _props$value === void 0 ? void 0 : _props$value.toString()) || '';
    onBeforeUnmount(function () {
      // focus:true unmount 不能触发 blur
      if (!focus.value) {
        emit('blur', props.value);
      }
    });
    return {
      container: container,
      focus: focus,
      currentValue: currentValue,
      onChange: onChange,
      onConfirm: onConfirm,
      addBlurListener: addBlurListener,
      removeBlurListener: removeBlurListener,
      getComponent: getComponent,
      getContainer: getContainer,
      doBlur: doBlur,
      onInputBlur: onInputBlur,
      onInputFocus: onInputFocus,
      onKeyboardClick: onKeyboardClick,
      onFakeInputClick: onFakeInputClick,
      focusFunc: focusFunc,
      renderPortal: renderPortal
    };
  },
  render: function render() {
    var onFakeInputClick = this.onFakeInputClick,
      renderPortal = this.renderPortal,
      placeholder = this.placeholder,
      disabled = this.disabled,
      editable = this.editable,
      moneyKeyboardAlign = this.moneyKeyboardAlign;
    var focus = this.focus,
      value = this.value;
    var preventKeyboard = disabled || !editable;
    var fakeInputCls = classnames("fake-input", {
      focus: focus,
      'fake-input-disabled': disabled
    });
    var fakeInputContainerCls = classnames('fake-input-container', {
      'fake-input-container-left': moneyKeyboardAlign === 'left'
    });
    return _createVNode("div", {
      "class": fakeInputContainerCls
    }, [value === '' && _createVNode("div", {
      "class": "fake-input-placeholder"
    }, [placeholder]), _createVNode("div", {
      "role": "textbox",
      "class": fakeInputCls,
      "onClick": preventKeyboard ? function () {} : onFakeInputClick
    }, [value]), renderPortal()]);
  }
});