import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { createVNode as _createVNode } from "vue";
import classnames from 'classnames';
import { defineComponent, ref } from 'vue';
import { IS_IOS } from '../../utils/exenv';
import TouchFeedback from '../../vmc-feedback';
var KeyboardItem = defineComponent({
  name: 'MKeyboardItem',
  props: {
    value: {
      type: [String, Number]
    },
    label: String,
    type: {
      type: String
    },
    prefixCls: {
      type: String,
      default: 'am-number-keyboard'
    },
    iconOnly: {},
    disabled: {
      type: Boolean,
      default: false
    }
  },
  render: function render() {
    var _this = this,
      _this$$slots$default,
      _this$$slots;
    var prefixCls = this.prefixCls,
      label = this.label,
      iconOnly = this.iconOnly;
    var type = this.type;
    var wrapCls = classnames("".concat(prefixCls, "-item"));
    return _createVNode(TouchFeedback, {
      "class": type,
      "activeClassName": "".concat(prefixCls, "-item-active")
    }, {
      default: function _default() {
        return [_createVNode("td", _objectSpread({
          "ref": "td",
          "onClick": function onClick(e) {
            _this.$emit('click', e, _this.value);
          },
          "class": wrapCls
        }, _this.$attrs), [(_this$$slots$default = (_this$$slots = _this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : _this$$slots$default.call(_this$$slots), iconOnly && _createVNode("i", {
          "class": "sr-only"
        }, [label])])];
      }
    });
  }
});
export default defineComponent({
  name: 'CustomKeyboard',
  props: {
    onClick: Function,
    open: {
      type: Boolean,
      default: false
    },
    prefixCls: {
      type: String
    },
    confirmLabel: [String, Object],
    backspaceLabel: {
      type: String
    },
    cancelKeyboardLabel: {
      type: String
    },
    wrapProps: {
      type: Object
    },
    header: [String, Object]
  },
  emits: ['click'],
  setup: function setup(_, _ref) {
    var emit = _ref.emit;
    var linkedInput = ref(null);
    var confirmDisabled = ref(null);
    var confirmKeyboardItem = ref(null);
    var onKeyboardClick = function onKeyboardClick(e) {
      var value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      e.stopImmediatePropagation();
      if (value === 'confirm' && confirmDisabled.value) {
        return null;
      } else {
        emit('click', value);
      }
    };
    var renderKeyboardItem = function renderKeyboardItem(item, index) {
      var KeyboardItem2 = KeyboardItem;
      return _createVNode(KeyboardItem2, {
        value: item,
        "onClick": onKeyboardClick,
        "key": "item-".concat(item, "-").concat(index)
      }, {
        default: function _default() {
          return [item];
        }
      });
    };
    var getAriaAttr = function getAriaAttr(label) {
      if (IS_IOS) {
        return {
          label: label,
          iconOnly: true
        };
      } else {
        return {
          role: 'button',
          'aria-label': label
        };
      }
    };
    return {
      linkedInput: linkedInput,
      confirmDisabled: confirmDisabled,
      confirmKeyboardItem: confirmKeyboardItem,
      onKeyboardClick: onKeyboardClick,
      renderKeyboardItem: renderKeyboardItem,
      getAriaAttr: getAriaAttr
    };
  },
  render: function render() {
    var onKeyboardClick = this.onKeyboardClick,
      prefixCls = this.prefixCls,
      confirmLabel = this.confirmLabel,
      backspaceLabel = this.backspaceLabel,
      cancelKeyboardLabel = this.cancelKeyboardLabel,
      getAriaAttr = this.getAriaAttr,
      renderKeyboardItem = this.renderKeyboardItem,
      open = this.open,
      wrapProps = this.wrapProps,
      header = this.header;
    var wrapperCls = classnames(_defineProperty(_defineProperty({}, "".concat(prefixCls, "-wrapper"), true), "".concat(prefixCls, "-wrapper-hide"), !open));
    return _createVNode("div", _objectSpread({
      "class": wrapperCls,
      "ref": "antmKeyboard"
    }, wrapProps), [header, _createVNode("table", null, [_createVNode("tbody", null, [_createVNode("tr", null, [['1', '2', '3'].map(function (item, index) {
      return (
        // tslint:disable-next-line:jsx-no-multiline-js
        renderKeyboardItem(item, index)
      );
    }), _createVNode(KeyboardItem, _objectSpread(_objectSpread({}, getAriaAttr(backspaceLabel)), {}, {
      type: 'keyboard-delete',
      rowSpan: 2,
      onClick: function onClick(e) {
        return onKeyboardClick(e, 'delete');
      }
    }), null)]), _createVNode("tr", null, [['4', '5', '6'].map(function (item, index) {
      return (
        // tslint:disable-next-line:jsx-no-multiline-js
        renderKeyboardItem(item, index)
      );
    })]), _createVNode("tr", null, [['7', '8', '9'].map(function (item, index) {
      return (
        // tslint:disable-next-line:jsx-no-multiline-js
        renderKeyboardItem(item, index)
      );
    }), _createVNode(KeyboardItem, {
      type: 'keyboard-confirm',
      rowSpan: 2,
      onClick: function onClick(e) {
        return onKeyboardClick(e, 'confirm');
      }
    }, {
      default: function _default() {
        return [confirmLabel];
      }
    })]), _createVNode("tr", null, [['.', '0'].map(function (item, index) {
      return (
        // tslint:disable-next-line:jsx-no-multiline-js
        renderKeyboardItem(item, index)
      );
    }), _createVNode(KeyboardItem, _objectSpread(_objectSpread({}, getAriaAttr(cancelKeyboardLabel)), {}, {
      type: 'keyboard-hide',
      onClick: function onClick(e) {
        return onKeyboardClick(e, 'hide');
      }
    }), null)])])])]);
  }
});