import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createVNode as _createVNode } from "vue";
import classNames from 'classnames';
import { computed, defineComponent, getCurrentInstance, inject, onBeforeUnmount, ref } from 'vue';
import Popover from '../../popover';
import toast from '../../toast';
import { filterHTMLAttrs } from '../../utils/dom';
import { isEmptySlot } from '../../utils/vnode';
import TouchFeedback from '../../vmc-feedback';
export var Brief = defineComponent({
  name: 'MListItemBrief',
  props: {
    prefixCls: {},
    role: {}
  },
  render: function render() {
    return _createVNode("div", {
      "class": "am-list-brief"
    }, [this.$slots.default()]);
  }
});
var Item = defineComponent({
  inheritAttrs: false,
  name: 'MListItem',
  props: {
    onClick: {},
    text: {
      type: Boolean,
      default: false
    },
    prefixCls: {
      default: 'am-list-item'
    },
    layout: {
      type: String
    },
    role: {
      type: String
    },
    platform: {
      type: String,
      default: 'iOS'
    },
    thumb: {
      type: [String, Object]
    },
    extra: {
      type: [String, Object]
    },
    control: {
      type: [String, Object]
    },
    extraPosition: {
      type: String,
      default: 'right'
    },
    activeStyle: {
      type: Object
    },
    multipleLine: {
      type: Boolean,
      default: false
    },
    error: {
      type: Boolean,
      default: false
    },
    errorMessage: {
      type: String
    },
    errorDisplayType: {
      type: String,
      default: 'text'
    },
    disabled: {
      type: Boolean,
      default: false
    },
    align: {
      type: String,
      default: 'middle'
    },
    wrap: {
      type: Boolean
    },
    arrow: {
      type: String
    },
    title: {
      type: [String, Object],
      default: ''
    },
    labelPosition: {
      type: String,
      default: 'left'
    },
    contentStyle: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    extraStyle: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    touchFeedback: {
      type: Boolean,
      default: false
    },
    required: {
      type: Boolean,
      default: false
    },
    suffix: {}
  },
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots;
    var debounceTimeout = ref(null);
    var coverRippleStyle = ref({
      display: 'none'
    });
    var rippleClicked = ref(false);
    var list = inject('list', undefined);
    var showErrorPopover = ref(false);
    var instance = getCurrentInstance();
    var actualError = computed(function () {
      var _ref2, _props$error;
      return (_ref2 = (_props$error = props.error) !== null && _props$error !== void 0 ? _props$error : instance.parent['error']) !== null && _ref2 !== void 0 ? _ref2 : false;
    });
    var actualErrorMessage = computed(function () {
      return props.errorMessage || instance.parent['errorMessage'];
    });
    var actualDisabled = computed(function () {
      return props.disabled;
    });
    var layout = computed(function () {
      return props.layout || (list === null || list === void 0 ? void 0 : list.layout) || 'horizontal';
    });
    var actualErrorDisplayType = computed(function () {
      var _props$errorDisplayTy;
      if (layout.value === 'vertical') {
        return 'text';
      }
      return (_props$errorDisplayTy = props.errorDisplayType) !== null && _props$errorDisplayTy !== void 0 ? _props$errorDisplayTy : instance.parent['errorDisplayType'];
    });
    var onClick = function onClick(ev) {
      ev.stopPropagation();
      var isAndroid = props.platform === 'android';
      if (isAndroid) {
        if (debounceTimeout.value) {
          clearTimeout(debounceTimeout.value);
          debounceTimeout.value = null;
        }
        var _Item = ev.currentTarget;
        var RippleWidth = Math.max(_Item.offsetHeight, _Item.offsetWidth);
        var ClientRect = ev.currentTarget.getBoundingClientRect();
        var pointX = ev.clientX - ClientRect.left - _Item.offsetWidth / 2;
        var pointY = ev.clientY - ClientRect.top - _Item.offsetWidth / 2;
        coverRippleStyle.value = {
          width: "".concat(RippleWidth, "px"),
          height: "".concat(RippleWidth, "px"),
          left: "".concat(pointX, "px"),
          top: "".concat(pointY, "px")
        };
        rippleClicked.value = true;
        debounceTimeout.value = setTimeout(function () {
          coverRippleStyle.value = {
            display: 'none'
          };
          rippleClicked.value = false;
        }, 1000);
      }
      emit('click');
    };
    var renderExtra = function renderExtra() {
      var extra = [];
      extra.push(actualError.value && actualErrorDisplayType.value !== 'text' ? _createVNode("div", {
        "class": "".concat(props.prefixCls, "-error-extra"),
        "onClick": function onClick(e) {
          e.stopPropagation();
          if (actualErrorMessage.value) {
            if (actualErrorDisplayType.value === 'toast') {
              toast.fail(actualErrorMessage.value);
            }
            if (actualErrorDisplayType.value === 'popover' && !showErrorPopover.value) {
              showErrorPopover.value = true;
            }
          }
          emit('error-click', e);
          emit('errorClick', e);
        }
      }, [actualErrorDisplayType.value === 'popover' ? _createVNode(Popover, {
        'value': showErrorPopover.value,
        "onUpdate:value": function onUpdateValue($event) {
          return showErrorPopover.value = $event;
        },
        "mask": false
      }, {
        content: function content() {
          return _createVNode(Popover.Item, null, {
            default: function _default() {
              return [actualErrorMessage.value];
            }
          });
        }
      }) : null]) : null);
      var hasExtra = !isEmptySlot(slots.extra) || props.extra || actualError.value && actualErrorMessage.value;
      if (hasExtra) {
        var _slots$extra;
        extra.push(_createVNode("div", {
          "style": props.extraStyle,
          "class": classNames("".concat(props.prefixCls, "-extra"), _defineProperty({}, "".concat(props.prefixCls, "-extra-text"), props.text))
        }, [((_slots$extra = slots.extra) === null || _slots$extra === void 0 ? void 0 : _slots$extra.call(slots)) || props.extra, actualErrorDisplayType.value === 'text' && actualError.value && actualErrorMessage.value ? _createVNode("div", null, [actualErrorMessage.value]) : null]));
      }
      return extra;
    };
    var renderThumb = function renderThumb() {
      var thumb = props.thumb,
        prefixCls = props.prefixCls;
      if (slots.thumb) {
        return _createVNode("div", {
          "class": "".concat(prefixCls, "-thumb")
        }, [slots.thumb()]);
      } else if (thumb) {
        return _createVNode("div", {
          "class": "".concat(prefixCls, "-thumb")
        }, [typeof thumb === 'string' ? _createVNode("img", {
          "src": thumb
        }, null) : thumb]);
      } else if (props.required) {
        return _createVNode("div", {
          "class": "".concat(prefixCls, "-required")
        }, null);
      } else {
        return null;
      }
    };
    var renderLabel = function renderLabel() {
      if (!isEmptySlot(slots.default)) {
        return _createVNode("div", {
          "class": "".concat(props.prefixCls, "-title"),
          "style": props.contentStyle
        }, [renderThumb(), slots.default()]);
      } else if (props.title) {
        return _createVNode("div", {
          "class": "".concat(props.prefixCls, "-title"),
          "style": props.contentStyle
        }, [renderThumb(), props.title]);
      } else {
        return renderThumb();
      }
    };
    var renderControl = function renderControl() {
      var _slots$control;
      return slots.control || props.control ? _createVNode("div", {
        "class": props.prefixCls + '-control'
      }, [((_slots$control = slots.control) === null || _slots$control === void 0 ? void 0 : _slots$control.call(slots)) || props.control]) : null;
    };
    onBeforeUnmount(function () {
      if (debounceTimeout.value) {
        clearTimeout(debounceTimeout.value);
        debounceTimeout.value = null;
      }
    });
    return {
      coverRippleStyle: coverRippleStyle,
      rippleClicked: rippleClicked,
      actualErrorDisplayType: actualErrorDisplayType,
      actualError: actualError,
      actualDisabled: actualDisabled,
      onClick: onClick,
      renderThumb: renderThumb,
      renderLabel: renderLabel,
      renderControl: renderControl,
      renderExtra: renderExtra,
      actualErrorMessage: actualErrorMessage,
      list: list,
      layout: layout
    };
  },
  render: function render() {
    var _ref3, _this$$slots$suffix, _this$$slots;
    var prefixCls = this.prefixCls,
      activeStyle = this.activeStyle,
      align = this.align,
      wrap = this.wrap,
      disabled = this.disabled,
      multipleLine = this.multipleLine,
      arrow = this.arrow,
      actualError = this.actualError;
    var coverRippleStyle = this.coverRippleStyle,
      rippleClicked = this.rippleClicked;
    var section = this.$parent['section'];
    var wrapCls = classNames(prefixCls, "".concat(prefixCls, "-label-").concat(this.labelPosition), (_ref3 = this.$attrs.class) !== null && _ref3 !== void 0 ? _ref3 : '', "".concat(prefixCls, "-").concat(this.layout), _defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty(_defineProperty({}, "".concat(prefixCls, "-disabled"), this.actualDisabled), "".concat(prefixCls, "-error"), actualError), "".concat(prefixCls, "-error-text"), actualError && this.actualErrorDisplayType === 'text'), "".concat(prefixCls, "-top"), align === 'top'), "".concat(prefixCls, "-middle"), align === 'middle'), "".concat(prefixCls, "-bottom"), align === 'bottom'), "".concat(prefixCls, "-section"), section), "".concat(prefixCls, "-extra-left"), this.extraPosition === 'left'), "".concat(prefixCls, "-extra-center"), this.extraPosition === 'center'), "".concat(prefixCls, "-extra-right"), this.extraPosition === 'right'));
    var rippleCls = classNames("".concat(prefixCls, "-ripple"), _defineProperty({}, "".concat(prefixCls, "-ripple-animate"), rippleClicked));
    var lineCls = classNames("".concat(prefixCls, "-line"), _defineProperty(_defineProperty({}, "".concat(prefixCls, "-line-multiple"), multipleLine), "".concat(prefixCls, "-line-wrap"), wrap));
    var arrowCls = classNames("".concat(prefixCls, "-arrow"), _defineProperty(_defineProperty(_defineProperty({}, "".concat(prefixCls, "-arrow-horizontal"), arrow === 'horizontal'), "".concat(prefixCls, "-arrow-vertical"), arrow === 'down' || arrow === 'up'), "".concat(prefixCls, "-arrow-vertical-up"), arrow === 'up'));
    var content = _createVNode("div", _objectSpread(_objectSpread({}, filterHTMLAttrs(this.$attrs)), {}, {
      "onClick": this.onClick,
      "class": wrapCls
    }), [_createVNode("div", {
      "class": lineCls
    }, [_createVNode("div", {
      "class": "".concat(prefixCls, "-content")
    }, [this.renderLabel(), this.renderControl(), this.renderExtra()]), arrow && _createVNode("div", {
      "class": arrowCls,
      "aria-hidden": "true"
    }, null), this.$slots.suffix || this.suffix ? _createVNode("div", {
      "class": this.prefixCls + '-suffix'
    }, [((_this$$slots$suffix = (_this$$slots = this.$slots).suffix) === null || _this$$slots$suffix === void 0 ? void 0 : _this$$slots$suffix.call(_this$$slots)) || this.suffix]) : null]), _createVNode("div", {
      "style": coverRippleStyle,
      "class": rippleCls
    }, null)]);
    var feedbackDisabled = disabled || !this.onClick || !this.touchFeedback || this.list && !this.list.touchFeedback;
    return _createVNode(TouchFeedback, {
      "disabled": feedbackDisabled,
      "activeStyle": activeStyle,
      "activeClassName": "".concat(prefixCls, "-active")
    }, {
      default: function _default() {
        return [content];
      }
    });
  }
});
export default Item;