import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { computed, defineComponent, ref, watch } from 'vue';
import { withNativeProps } from '../utils/native-props';
import { renderToContainer } from '../utils/render-to-container';
import { useLockScroll } from '../utils/use-lock-scroll';
import { useShouldRender } from '../utils/use-should-render';
import { withStopPropagation } from '../utils/with-stop-propagation';
var classPrefix = "adm-mask";
var opacityRecord = {
  default: 0.55,
  thin: 0.35,
  thick: 0.75
};
export var Mask = defineComponent({
  name: 'MMask',
  props: {
    open: {
      type: Boolean,
      default: false
    },
    onMaskClick: Function,
    destroyOnClose: Boolean,
    forceRender: Boolean,
    disableBodyScroll: Boolean,
    color: {
      type: String,
      default: 'black'
    },
    opacity: {
      type: [String, Number],
      default: 'default'
    },
    getContainer: [Object, Function],
    afterShow: Function,
    afterClose: Function
  },
  setup: function setup(props) {
    var wrapperRef = ref(null);
    useLockScroll(wrapperRef, props.open && props.disableBodyScroll);
    var background = computed(function () {
      var _opacityRecord$props$;
      var opacity = (_opacityRecord$props$ = opacityRecord[props.opacity]) !== null && _opacityRecord$props$ !== void 0 ? _opacityRecord$props$ : props.opacity;
      var rgb = props.color === 'white' ? '255, 255, 255' : '0, 0, 0';
      return "rgba(".concat(rgb, ", ").concat(opacity, ")");
    });
    var active = ref(props.open);
    var opacity = ref(0);
    var intervalRef = ref();
    watch(function () {
      return props.open;
    }, function (open) {
      if (intervalRef.value) {
        clearInterval(intervalRef.value);
        opacity.value = open ? 0 : 1;
      }
      intervalRef.value = setInterval(function () {
        if (open) {
          if (opacity.value >= 1) {
            onEnd();
          } else {
            opacity.value += 0.01;
          }
        } else {
          if (opacity.value <= 0) {
            onEnd();
          } else {
            opacity.value -= 0.01;
          }
        }
      }, 2);
    }, {
      immediate: true
    });
    var onEnd = function onEnd() {
      clearInterval(intervalRef.value);
      intervalRef.value = undefined;
      active.value = props.open;
      if (props.open) {
        var _props$afterShow;
        (_props$afterShow = props.afterShow) === null || _props$afterShow === void 0 ? void 0 : _props$afterShow.call(props);
      } else {
        var _props$afterClose;
        (_props$afterClose = props.afterClose) === null || _props$afterClose === void 0 ? void 0 : _props$afterClose.call(props);
      }
    };
    var shouldRender = useShouldRender(active.value, props.forceRender, props.destroyOnClose);
    return {
      renderToContainer: renderToContainer,
      background: background,
      wrapperRef: wrapperRef,
      shouldRender: shouldRender,
      active: active,
      opacity: opacity
    };
  },
  render: function render() {
    var _this$$slots$default, _this$$slots;
    var display = this.open ? 'unset' : this.active ? 'unset' : 'none';
    var node = withStopPropagation(['click'], withNativeProps(this.$props, _createVNode("div", {
      "class": classPrefix,
      "ref": this.wrapperRef,
      "style": _objectSpread(_objectSpread({}, this.$attrs.style || {}), {}, {
        opacity: this.opacity,
        background: this.background,
        display: display
      })
    }, [this.onMaskClick && _createVNode("div", {
      "class": "".concat(classPrefix, "-aria-button"),
      "role": "button",
      "onClick": this.onMaskClick
    }, null), _createVNode("div", {
      "class": "".concat(classPrefix, "-content")
    }, [(_this$$slots$default = (_this$$slots = this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : _this$$slots$default.call(_this$$slots)])])));
    return renderToContainer(this.getContainer, node);
  }
});