import _typeof from "@babel/runtime/helpers/esm/typeof";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import { computed, onBeforeUpdate, ref, watch } from 'vue';
import { getOptionProperty } from '../utils/option';
import { getNodeText, isEmptySlot } from '../utils/vnode';
import { unwrapFragment } from '../utils/vue';
import { useBaseInputComponent } from './base-input-component';
import { simpleFormComponentProps } from './simple-form-component';
var _getFlattenOptions = function getFlattenOptions(options) {
  return options.reduce(function (acc, option) {
    var children = option.children || [];
    acc.push(option);
    if (children.length) {
      acc.push.apply(acc, _toConsumableArray(_getFlattenOptions(children)));
    }
    return acc;
  }, []);
};
export var optionsBasedComponentProps = _objectSpread(_objectSpread({}, simpleFormComponentProps), {}, {
  /**
   * 是否可搜索
   */
  searchable: {
    type: Boolean,
    default: false
  },
  filterOption: Function,
  /**
   * 选项对象中作为标签的属性名称
   */
  labelProperty: {
    type: [String, Function],
    default: 'label'
  },
  /**
   * 选项对象中作为值的属性名称
   */
  valueProperty: {
    type: [String, Function],
    default: 'value'
  },
  /**
   * 选项数据
   */
  options: {
    type: Array
  }
});
export var useOptionsBaseComponent = function useOptionsBaseComponent(props, _ref, form) {
  var emit = _ref.emit,
    attrs = _ref.attrs,
    slots = _ref.slots;
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {
    defaultValue: undefined,
    propName: 'value'
  };
  var _useBaseInputComponen = useBaseInputComponent(props, {
      emit: emit,
      attrs: attrs,
      slots: slots
    }, form, options),
    isDisabled = _useBaseInputComponen.isDisabled,
    setStateValue = _useBaseInputComponen.setStateValue,
    stateValue = _useBaseInputComponen.stateValue,
    isReadonly = _useBaseInputComponen.isReadonly;
  var searchKeyword = ref('');
  watch(function () {
    return props.searchText;
  }, function (searchText) {
    searchKeyword.value = searchText;
  });
  watch(function () {
    return searchKeyword.value;
  }, function (keyword) {
    emit('search', keyword);
  });
  var flattenOptions = computed(function () {
    var _props$options;
    return _getFlattenOptions((_props$options = props.options) !== null && _props$options !== void 0 ? _props$options : []);
  });
  var valueOptionMap = computed(function () {
    return Object.fromEntries(flattenOptions.value.map(function (item) {
      return [getOptionProperty(item, props.valueProperty), item];
    }));
  });
  var valueParentIdMap = computed(function () {
    var res = {};
    flattenOptions.value.filter(function (it) {
      var _it$children;
      return (_it$children = it.children) === null || _it$children === void 0 ? void 0 : _it$children.length;
    }).forEach(function (parent) {
      parent.children.forEach(function (child) {
        res[getOptionProperty(child, props.valueProperty)] = getOptionProperty(parent, props.valueProperty);
      });
    });
    return res;
  });
  var optionFilter = function optionFilter(item) {
    if (props.filterOption) {
      return props.filterOption(item);
    }
    var label = getOptionProperty(item, props.labelProperty);
    if (_typeof(label) === 'object') {
      label = getNodeText(label) || '';
    }
    return !searchKeyword.value || label.includes(searchKeyword.value);
  };
  var filteredValues = computed(function () {
    var directMatchValues = flattenOptions.value.filter(optionFilter).map(function (it) {
      return getOptionProperty(it, props.valueProperty);
    });
    var parentValues = [];
    directMatchValues.forEach(function (value) {
      var currentValue = value;
      while (valueParentIdMap.value[currentValue]) {
        parentValues.push(valueParentIdMap.value[currentValue]);
        currentValue = valueParentIdMap.value[currentValue];
      }
    });
    return [].concat(_toConsumableArray(directMatchValues), parentValues);
  });
  var getOptions = function getOptions() {
    return getResolvedOptions(props.options);
  };
  var getResolvedOptions = function getResolvedOptions(options) {
    if (options) {
      return options.filter(optionFilter).map(function (item) {
        return _objectSpread(_objectSpread({}, item), {}, {
          label: getOptionProperty(item, props.labelProperty),
          value: getOptionProperty(item, props.valueProperty)
        });
      });
    } else {
      return [];
    }
  };
  var setProps = function setProps() {
    if (!isEmptySlot(slots.default)) {
      unwrapFragment(slots.default()).forEach(function (node) {
        if (node.props.disabled === undefined) {
          node.props.disabled = isDisabled.value;
        }
        if (node.props.readonly === undefined) {
          node.props.readonly = isReadonly.value;
        }
      });
    }
  };
  onBeforeUpdate(function () {
    setProps();
  });
  setProps();
  return {
    getOptions: getOptions,
    isReadonly: isReadonly,
    isDisabled: isDisabled,
    searchKeyword: searchKeyword,
    valueParentIdMap: valueParentIdMap,
    stateValue: stateValue,
    setStateValue: setStateValue,
    flattenOptions: flattenOptions,
    valueOptionMap: valueOptionMap,
    filteredValues: filteredValues
  };
};