import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import { createVNode as _createVNode } from "vue";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
var _excluded = ["popupPrefixCls", "itemStyle", "indicatorStyle", "okText", "cancelText", "extra", "cascade", "prefixCls", "pickerPrefixCls", "data", "cols"];
import { arrayTreeFilter } from '../../utils/array';
import { cloneVNode, defineComponent, provide, reactive, ref, watch } from 'vue';
import { setProps } from '../../utils/vnode';
import RMCCascader from '../../vmc-cascader/cascader';
import RMCPopupCascader from '../../vmc-cascader/popup';
import RMCMultiPicker from '../../vmc-picker/multi-picker';
import RMCPicker from '../../vmc-picker/picker';
export var PickerMixin = function PickerMixin(isView) {
  return defineComponent({
    name: 'MPicker',
    props: {
      placeholder: {
        type: String,
        default: ''
      },
      cancelText: {
        type: String,
        default: '取消'
      },
      okText: {
        type: String,
        default: '确定'
      },
      prefixCls: {
        type: String,
        default: 'am-picker'
      },
      triggerType: {
        type: String,
        default: 'click'
      },
      pickerPrefixCls: {
        type: String,
        default: 'am-picker-col'
      },
      popupPrefixCls: {
        type: String,
        default: 'am-picker-popup'
      },
      title: {
        type: [String, Object],
        default: ''
      },
      data: {},
      cascade: {
        type: Boolean,
        default: true
      },
      value: {
        type: Array
      },
      format: {
        type: Function,
        default: function _default(values) {
          if (values.length > 0 && typeof values[0] !== 'string') {
            return values;
          }
          return values.join(',');
        }
      },
      cols: {
        type: Number,
        default: 3
      },
      extra: {},
      onChange: {},
      itemStyle: {},
      indicatorStyle: {},
      open: {
        type: Boolean,
        default: false
      }
    },
    setup: function setup(props, _ref) {
      var _props$value;
      var emit = _ref.emit,
        slots = _ref.slots;
      var state = reactive({
        open: props.open
      });
      watch(function () {
        return props.open;
      }, function (v) {
        state.open = v;
      });
      watch(function () {
        return state.open;
      }, function (v) {
        return emit('update:open', v);
      });
      var currentValue = ref((_props$value = props.value) !== null && _props$value !== void 0 ? _props$value : []);
      var popupProps = ref(null);
      watch(function () {
        return props.value;
      }, function (v) {
        if (v && v !== currentValue.value) {
          currentValue.value = _toConsumableArray(v);
        }
      }, {
        immediate: true
      });
      var getSel = function getSel() {
        var value = currentValue.value || [];
        var treeChildren;
        var data = props.data;
        if (props.cascade) {
          treeChildren = arrayTreeFilter(data, function (c, level) {
            return c.value === value[level];
          });
        } else {
          treeChildren = value.map(function (v, i) {
            return data[i].filter(function (d) {
              return d.value === v;
            })[0];
          });
        }
        var extra = props.format && props.format(treeChildren.map(function (v) {
          return v.label;
        }));
        if (Array.isArray(extra)) {
          return extra[0];
        }
        return extra;
      };
      var getPickerCol = function getPickerCol() {
        var data = props.data,
          pickerPrefixCls = props.pickerPrefixCls,
          itemStyle = props.itemStyle,
          indicatorStyle = props.indicatorStyle;
        return data.map(function (col, index) {
          return _createVNode(RMCPicker, {
            "key": index,
            "prefixCls": pickerPrefixCls,
            "style": {
              flex: 1
            },
            "data": col,
            "itemStyle": itemStyle,
            "indicatorStyle": indicatorStyle
          }, null);
        });
      };
      var onOk = function onOk(v) {
        emit('update:value', _toConsumableArray(currentValue.value));
        state.open = false;
      };
      var setScrollValue = function setScrollValue(v) {
        currentValue.value = v;
      };
      var setCasecadeScrollValue = function setCasecadeScrollValue(v) {
        // 级联情况下保证数据正确性，滚动过程中只有当最后一级变化时才变更数据
        if (v && currentValue.value) {
          var length = currentValue.value.length;
          if (length === v.length && currentValue.value[length - 1] === v[length - 1]) {
            return;
          }
        }
        setScrollValue(v);
      };
      var onPickerChange = function onPickerChange(v) {
        setScrollValue(v);
        emit('pickerChange', v);
      };
      var onCancel = function onCancel() {
        var _props$value2;
        state.open = false;
        currentValue.value = (_props$value2 = props.value) !== null && _props$value2 !== void 0 ? _props$value2 : [];
      };
      var getPlaceholder = function getPlaceholder() {
        return props.placeholder || '';
      };
      provide('store', {
        onOk: null
      });
      return {
        currentValue: currentValue,
        getPickerCol: getPickerCol,
        getPlaceholder: getPlaceholder,
        getSel: getSel,
        onCancel: onCancel,
        onOk: onOk,
        onPickerChange: onPickerChange,
        popupProps: popupProps,
        setCasecadeScrollValue: setCasecadeScrollValue,
        setScrollValue: setScrollValue,
        state: state
      };
    },
    render: function render() {
      var _this = this;
      var _this$$props = this.$props,
        popupPrefixCls = _this$$props.popupPrefixCls,
        itemStyle = _this$$props.itemStyle,
        indicatorStyle = _this$$props.indicatorStyle,
        okText = _this$$props.okText,
        cancelText = _this$$props.cancelText,
        extra = _this$$props.extra,
        cascade = _this$$props.cascade,
        prefixCls = _this$$props.prefixCls,
        pickerPrefixCls = _this$$props.pickerPrefixCls,
        data = _this$$props.data,
        cols = _this$$props.cols,
        restProps = _objectWithoutProperties(_this$$props, _excluded);
      var _cascader;
      var popupMoreProps = {};
      if (cascade) {
        _cascader = _createVNode(RMCCascader, {
          "prefixCls": prefixCls,
          "pickerPrefixCls": pickerPrefixCls,
          "data": data,
          "cols": cols,
          'value': _this.currentValue,
          "onUpdate:value": function onUpdateValue($event) {
            return _this.currentValue = $event;
          },
          "onChange": this.onPickerChange,
          "onScrollChange": this.setCasecadeScrollValue,
          "pickerItemStyle": itemStyle,
          "indicatorStyle": indicatorStyle
        }, null);
      } else {
        _cascader = _createVNode(RMCMultiPicker, {
          "style": {
            flexDirection: 'row',
            alignItems: 'center'
          },
          "prefixCls": prefixCls,
          'value': _this.currentValue,
          "onUpdate:value": function onUpdateValue($event) {
            return _this.currentValue = $event;
          },
          "onScrollChange": this.setScrollValue
        }, {
          default: function _default() {
            return [_this.getPickerCol()];
          }
        });
        popupMoreProps = {
          pickerValueProp: 'value',
          pickerValueChangeProp: 'onValueChange'
        };
      }
      if (isView) {
        return _cascader;
      }
      var props = _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, this.popupProps), restProps), popupMoreProps), {}, {
        prefixCls: popupPrefixCls,
        open: this.state.open,
        cascader: _cascader,
        cancelText: cancelText,
        okText: okText,
        onOk: this.onOk,
        onCancel: this.onCancel
      });
      var childExtra = this.getSel() || extra || this.getPlaceholder() || '';
      return _createVNode(RMCPopupCascader, props, {
        cascader: function cascader() {
          return _cascader;
        },
        default: function _default() {
          var _this$$slots$default, _this$$slots, _this$$slots$default$;
          return (_this$$slots$default = (_this$$slots = _this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : (_this$$slots$default$ = _this$$slots$default.call(_this$$slots)) === null || _this$$slots$default$ === void 0 ? void 0 : _this$$slots$default$.map(function (child) {
            var node = cloneVNode(child);
            setProps(node, {
              touchFeedback: true,
              extra: childExtra,
              arrow: 'horizontal',
              onClick: function onClick() {
                _this.state.open = true;
              }
            });
            return node;
          });
        }
      });
    }
  });
};
export default PickerMixin(false);