import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { Drawer } from 'ant-design-vue';
import classNames from 'classnames';
import { defineComponent, inject, ref } from 'vue';
import Icon from '../../icon';
import { useBaseInputComponent } from '../../mixins/base-input-component';
import { pureInputComponentProps } from '../../mixins/pure-input-component';
import Touchable from '../../vmc-feedback/feedback';
import omit from "omit.js";
import { getZIndex } from "../../utils/zindex-manage";
export default defineComponent({
  name: 'MPopup',
  inheritAttrs: false,
  props: _objectSpread(_objectSpread({}, pureInputComponentProps), {}, {
    onCancel: {},
    onOk: {},
    open: {
      type: Boolean,
      default: false
    },
    cancelText: {
      type: String,
      default: '取消'
    },
    showCancel: {
      type: Boolean,
      default: false
    },
    cancelButton: {
      type: Object
    },
    showOk: {
      type: Boolean,
      default: true
    },
    title: {
      type: [String, Object],
      default: ''
    },
    prefixCls: {
      type: String,
      default: 'am-popup'
    },
    height: {
      type: String
    },
    width: {
      type: String
    },
    placement: {
      type: String,
      default: 'bottom'
    },
    showTitle: {
      type: Boolean,
      default: true
    },
    confirmLoading: {
      type: Boolean
    },
    getContainer: {
      type: [Object, String, Boolean],
      default: 'body'
    },
    loadingText: {
      type: String
    },
    maskClosable: {
      type: Boolean,
      default: true
    },
    okText: {
      type: String,
      default: '确定'
    }
  }),
  install: null,
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      attrs = _ref.attrs,
      slots = _ref.slots;
    var form = inject('list', undefined);
    var _useBaseInputComponen = useBaseInputComponent(props, {
        emit: emit,
        attrs: attrs,
        slots: slots
      }, form, {
        propName: 'open',
        defaultValue: props.open
      }),
      stateValue = _useBaseInputComponen.stateValue,
      cssStyle = _useBaseInputComponen.cssStyle,
      getDefaultSlot = _useBaseInputComponen.getDefaultSlot,
      isDisabled = _useBaseInputComponen.isDisabled,
      isReadonly = _useBaseInputComponen.isReadonly;
    var onCancel = function onCancel(e) {
      if (props.onCancel) {
        emit('cancel', e);
      } else {
        stateValue.value = false;
      }
    };
    var onOk = function onOk(e) {
      if (props.onOk) {
        emit('ok', e);
      } else {
        stateValue.value = false;
      }
    };
    var getProps = function getProps() {
      var open = isDisabled.value ? false : stateValue.value;
      return {
        block: props.block,
        cancelButton: props.cancelButton,
        cancelText: props.cancelText,
        confirmLoading: props.confirmLoading,
        getContainer: props.getContainer,
        loadingText: props.loadingText,
        maskClosable: props.maskClosable,
        prefixCls: props.prefixCls,
        okText: props.okText,
        onCancel: props.onCancel,
        onOk: props.onOk,
        showCancel: props.showCancel,
        showOk: props.showOk,
        showTitle: props.showTitle,
        value: props.value,
        title: renderHeader(),
        height: props.height || 'auto',
        width: props.width || 'auto',
        disabled: isDisabled.value,
        placement: props.placement,
        open: open
      };
    };
    var getListeners = function getListeners() {
      return {
        onClose: function onClose() {
          stateValue.value = false;
          emit('cancel');
        }
      };
    };
    var renderHeader = function renderHeader() {
      return props.showTitle ? _createVNode("div", {
        "class": "".concat(props.prefixCls, "-title-wrap")
      }, [renderCancel(), _createVNode("div", {
        "class": "".concat(props.prefixCls, "-item ").concat(props.prefixCls, "-title")
      }, [props.title]), renderOk()]) : null;
    };
    var renderCancel = function renderCancel() {
      return props.showCancel && !props.confirmLoading ? _createVNode(Touchable, {
        "disabled": props.confirmLoading,
        "activeClassName": "".concat(props.prefixCls, "-item-active")
      }, {
        default: function _default() {
          return [_createVNode("div", {
            "onClick": function onClick(e) {
              onCancel(e);
            },
            "class": classNames("".concat(props.prefixCls, "-item ").concat(props.prefixCls, "-header-left"))
          }, [props.cancelText])];
        }
      }) : null;
    };
    var renderOk = function renderOk() {
      return props.showOk ? _createVNode(Touchable, {
        "disabled": props.confirmLoading,
        "activeClassName": "".concat(props.prefixCls, "-item-active")
      }, {
        default: function _default() {
          return [_createVNode("div", {
            "onClick": function onClick(e) {
              if (!props.confirmLoading) {
                onOk(e);
              }
            },
            "class": classNames("".concat(props.prefixCls, "-item ").concat(props.prefixCls, "-header-right"), _defineProperty({}, "".concat(props.prefixCls, "-item-disabled"), props.confirmLoading))
          }, [props.confirmLoading ? _createVNode(Icon, {
            "type": 'loading'
          }, null) : undefined, props.confirmLoading ? props.loadingText : props.okText])];
        }
      }) : null;
    };
    var drawerRef = ref();
    var setDrawerRef = function setDrawerRef(el) {
      drawerRef.value = el;
    };
    var calcContentHeight = function calcContentHeight() {
      // todo
    };
    return {
      getProps: getProps,
      slots: slots,
      getDefaultSlot: getDefaultSlot,
      cssStyle: cssStyle,
      calcContentHeight: calcContentHeight,
      setDrawerRef: setDrawerRef,
      stateValue: stateValue,
      getListeners: getListeners,
      isReadonly: isReadonly,
      isDisabled: isDisabled
    };
  },
  render: function render() {
    var props = omit(_objectSpread(_objectSpread(_objectSpread({}, this.getListeners()), this.getProps()), {}, {
      closable: false,
      style: this.cssStyle,
      open: this.stateValue,
      maskClosable: this.confirmLoading ? false : this.maskClosable
    }), ['cancelText', 'loadingText', 'okText', 'showOk', 'showTitle', 'block', 'confirmLoading', 'showCancel']);
    this.calcContentHeight();
    return _createVNode(Drawer, _objectSpread(_objectSpread({}, props), {}, {
      "ref": this.setDrawerRef,
      "zIndex": getZIndex(),
      "push": {
        distance: 0
      }
    }), this.slots);
  }
});