import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createVNode as _createVNode } from "vue";
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent, getCurrentInstance, inject, onMounted } from 'vue';
import List from '../../list';
import { optionsBasedComponentProps, useOptionsBaseComponent } from '../../mixins/options-based-component';
import SearchBar from '../../search-bar/src';
import { filterHTMLAttrs } from '../../utils/dom';
import RadioItem from './radio-item';
export default defineComponent({
  name: 'MRadioList',
  inheritAttrs: false,
  props: _objectSpread(_objectSpread({}, optionsBasedComponentProps), {}, {
    value: {},
    title: {
      type: String
    },
    onChange: [Function, Array],
    maxHeightPercentage: {
      type: Number
    },
    iconType: {
      type: String,
      default: 'radio'
    }
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var instance = getCurrentInstance();
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        slots: slots,
        attrs: attrs
      }, form),
      getOptions = _useOptionsBaseCompon.getOptions,
      stateValue = _useOptionsBaseCompon.stateValue,
      setStateValue = _useOptionsBaseCompon.setStateValue,
      isReadonly = _useOptionsBaseCompon.isReadonly,
      searchKeyword = _useOptionsBaseCompon.searchKeyword,
      isDisabled = _useOptionsBaseCompon.isDisabled;
    var renderOptions = function renderOptions() {
      var options = getOptions();
      if (options) {
        return options.map(function (option, index) {
          var optionProps = {};
          if (index === 0) {
            _extends(optionProps, {
              error: props.error,
              errorDisplayType: props.errorDisplayType,
              errorMessage: props.errorMessage
            });
          }
          return _createVNode(RadioItem, _objectSpread(_objectSpread({}, optionProps), {}, {
            "iconType": option.loading ? 'loading' : props.iconType,
            "disabled": option.disabled || isDisabled.value,
            "readOnly": isReadonly.value,
            "value": stateValue.value === option.value,
            "key": option.value,
            "onChange": function onChange(checkState) {
              _onChange(checkState, option.value);
            }
          }), {
            default: function _default() {
              return [option.label];
            }
          });
        });
      } else {
        return [];
      }
    };
    var _onChange = function _onChange(checkState, value) {
      setStateValue(value);
    };
    var renderSearch = function renderSearch() {
      return props.searchable ? _createVNode(SearchBar, _objectSpread({
        "value": searchKeyword.value
      }, _defineProperty({}, 'onUpdate:value', function onUpdateValue(v) {
        searchKeyword.value = v;
      })), null) : null;
    };
    onMounted(function () {
      if (props.maxHeightPercentage) {
        var windowHeight = document.body.clientHeight;
        var maxHeight = props.maxHeightPercentage;
        if (instance.vnode.el.clientHeight > windowHeight * maxHeight) {
          instance.vnode.el.style.height = windowHeight * maxHeight + 'px';
        }
      }
    });
    return {
      renderOptions: renderOptions,
      renderSearch: renderSearch
    };
  },
  render: function render() {
    var _this = this;
    return _createVNode(List, _objectSpread(_objectSpread({}, filterHTMLAttrs(this.$attrs)), {}, {
      "required": this.required,
      "title": this.title
    }), {
      default: function _default() {
        return [_this.renderSearch(), _this.renderOptions()];
      }
    });
  }
});