import { Fragment as _Fragment } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createTextVNode as _createTextVNode, createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { computed, defineComponent, inject, ref } from 'vue';
import List from '../../list';
import { optionsBasedComponentProps, useOptionsBaseComponent } from '../../mixins/options-based-component';
import Popup from '../../popup';
import RadioList from './radio-list';
import { isNotNull, isNull } from '../../utils/util';
import classNames from 'classnames';
export default defineComponent({
  name: 'MRadioPopupList',
  props: _objectSpread(_objectSpread({}, optionsBasedComponentProps), {}, {
    title: {
      type: [String, Object]
    },
    placeholder: {
      type: String
    },
    clearable: {
      type: Boolean,
      default: false
    }
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var popupOpen = ref(false);
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        slots: slots,
        attrs: attrs
      }, form),
      getOptions = _useOptionsBaseCompon.getOptions,
      isDisabled = _useOptionsBaseCompon.isDisabled,
      isReadonly = _useOptionsBaseCompon.isReadonly,
      stateValue = _useOptionsBaseCompon.stateValue;
    var optionText = computed(function () {
      var options = getOptions();
      var value = stateValue.value;
      var selectedOption = options.find(function (it) {
        return value === it.value;
      });
      return selectedOption && selectedOption.label;
    });
    var onCancel = function onCancel() {
      closePopup();
    };
    var onChange = function onChange(value) {
      stateValue.value = value;
      popupOpen.value = false;
    };
    var onClick = function onClick() {
      if (!isDisabled.value && !isReadonly.value) {
        popupOpen.value = true;
      }
    };
    var onClear = function onClear() {
      emit('clear');
      emit('update:value', null);
      closePopup();
    };
    var closePopup = function closePopup() {
      popupOpen.value = false;
    };
    return {
      onClick: onClick,
      onChange: onChange,
      onCancel: onCancel,
      onClear: onClear,
      getOptions: getOptions,
      stateValue: stateValue,
      closePopup: closePopup,
      optionText: optionText,
      popupOpen: popupOpen,
      isDisabled: isDisabled
    };
  },
  render: function render() {
    var _this = this;
    var listProps = _objectSpread(_objectSpread(_objectSpread({}, this.$attrs), this.$props), {}, {
      options: this.getOptions()
    });
    listProps.title = undefined;
    var cancelButton = _createVNode("div", {
      "onClick": this.onClear,
      "class": "am-popup-item am-popup-header-left"
    }, [_createTextVNode("\u6E05\u9664")]);
    var optionText = this.optionText,
      placeholder = this.placeholder,
      stateValue = this.stateValue,
      closePopup = this.closePopup,
      title = this.title,
      clearable = this.clearable,
      onClick = this.onClick,
      readOnly = this.readOnly,
      isDisabled = this.isDisabled,
      disabled = this.disabled;
    var slots = {
      control: function control() {
        return _createVNode("span", {
          "class": _defineProperty({}, "am-list-item-placeholder", isNull(stateValue))
        }, [isNotNull(stateValue) ? optionText : placeholder]);
      },
      default: function _default() {
        return _createVNode("span", null, [title]);
      }
    };
    return _createVNode(_Fragment, null, [_createVNode(List.Item, {
      "onClick": onClick,
      "text": !!optionText,
      "arrow": 'horizontal',
      "error": this.error,
      "errorMessage": this.errorMessage,
      "errorDisplayType": this.errorDisplayType,
      "style": this.$attrs.style,
      "class": classNames(this.$attrs.class, 'am-radio-popup-list'),
      "required": this.required,
      "touchFeedback": !readOnly && !disabled,
      "disabled": isDisabled
    }, slots), _createVNode(Popup, {
      "open": isDisabled ? false : this.popupOpen,
      "showCancel": clearable,
      "cancelButton": cancelButton,
      "title": title,
      "onOk": closePopup,
      "onCancel": closePopup
    }, {
      default: function _default() {
        return [_createVNode(RadioList, _objectSpread(_objectSpread({}, listProps), {}, {
          "maxHeightPercentage": 0.7,
          "onChange": _this.onChange
        }), null)];
      }
    })]);
  }
});