import { createVNode as _createVNode } from "vue";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import classNames from 'classnames';
import { defineComponent, inject } from 'vue';
import Grid from '../grid';
import { creatFormComponentProps, useFormComponent } from '../mixins/form-component';
import { optionsBasedComponentProps, useOptionsBaseComponent } from '../mixins/options-based-component';
import Space from '../space';
import { getOptionProperty } from '../utils/option';
import { CheckMark } from './check-mark';
export default defineComponent({
  name: 'MSelector',
  props: _objectSpread(_objectSpread(_objectSpread({}, creatFormComponentProps()), optionsBasedComponentProps), {}, {
    prefixCls: {
      type: String,
      default: 'am-selector'
    },
    columns: {},
    multiple: {
      type: Boolean,
      default: false
    },
    disabled: {
      type: Boolean
    },
    defaultValue: {
      type: [Array, String, Number],
      default: function _default() {
        return [];
      }
    },
    onChange: {
      type: Function
    },
    showCheckMark: {
      type: Boolean,
      default: true
    }
  }),
  emit: ['change'],
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var onChange = function onChange(v) {
      if (props.multiple) {
        emit('change', v, props.options.filter(function (option) {
          return v.includes(getOptionProperty(option, props.valueProperty));
        }));
      } else {
        emit('change', v, props.options.find(function (option) {
          return v === getOptionProperty(option, props.valueProperty);
        }));
      }
    };
    var _useFormComponent = useFormComponent(props, {
        emit: emit,
        onChange: onChange
      }),
      currentValue = _useFormComponent.currentValue;
    var form = inject('list', undefined);
    var _useOptionsBaseCompon = useOptionsBaseComponent(props, {
        emit: emit,
        slots: slots,
        attrs: attrs
      }, form),
      getOptions = _useOptionsBaseCompon.getOptions;
    var setValue = function setValue(v) {
      currentValue.value = v;
    };
    return {
      getOptions: getOptions,
      currentValue: currentValue,
      setValue: setValue
    };
  },
  render: function render() {
    var prefixCls = this.prefixCls,
      columns = this.columns,
      multiple = this.multiple,
      setValue = this.setValue,
      showCheckMark = this.showCheckMark,
      getOptions = this.getOptions,
      currentValue = this.currentValue;
    var items = getOptions().map(function (option) {
      var active = multiple ? (currentValue || []).includes(option.value) : option.value === currentValue;
      var disabled = option.disabled;
      var itemCls = classNames("".concat(prefixCls, "-item"), _defineProperty(_defineProperty(_defineProperty({}, "".concat(prefixCls, "-item-active"), active && !multiple), "".concat(prefixCls, "-item-multiple-active"), active && multiple), "".concat(prefixCls, "-item-disabled"), disabled));
      return _createVNode("div", {
        "key": option.value,
        "class": itemCls,
        "onClick": function onClick() {
          if (disabled) {
            return;
          }
          if (multiple) {
            var val = active ? currentValue.filter(function (v) {
              return v !== option.value;
            }) : [].concat(_toConsumableArray(currentValue), [option.value]);
            setValue(val);
          } else {
            var _val = active ? undefined : option.value;
            setValue(_val);
          }
        },
        "role": "option",
        "aria-selected": active && !multiple || active && multiple
      }, [option.label, option.description && _createVNode("div", {
        "class": "".concat(prefixCls, "-item-description")
      }, [option.description]), active && showCheckMark && _createVNode("div", {
        "class": "".concat(prefixCls, "-check-mark-wrapper")
      }, [_createVNode(CheckMark, null, null)])]);
    });
    return _createVNode("div", {
      "class": prefixCls,
      "role": "listbox"
    }, [columns ? _createVNode(Grid, {
      "cols": columns
    }, {
      default: function _default() {
        return [items];
      }
    }) : _createVNode(Space, {
      "wrap": true
    }, {
      default: function _default() {
        return [items];
      }
    })]);
  }
});