import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createVNode as _createVNode } from "vue";
import classNames from 'classnames';
import { computed, defineComponent, getCurrentInstance, isVNode } from 'vue';
import Icon from '../../icon';
function isString(str) {
  return typeof str === 'string';
}
var statusIcon = {
  finish: 'check-circle',
  error: 'cross-circle-o',
  wait: 'ellipsis'
};
export default defineComponent({
  name: 'MStep',
  props: {
    prefixCls: {
      type: String,
      default: 'am-steps'
    },
    wrapperStyle: {},
    itemWidth: {
      type: [Number, String]
    },
    status: {
      type: String
    },
    iconPrefix: {
      type: String,
      default: 'ant'
    },
    /**
     * 图标类型，仅支持的图标名称
     */
    icon: {
      type: [String, Object]
    },
    adjustMarginRight: {
      type: [Number, String]
    },
    stepNumber: {
      type: Number
    },
    description: {
      type: String
    },
    title: {
      type: String
    },
    progressDot: {}
  },
  setup: function setup(props, _ref) {
    var slots = _ref.slots;
    var instance = getCurrentInstance();
    var iconSize = computed(function () {
      if (instance.parent.props.size === 'small') {
        return 18;
      } else {
        return 22;
      }
    });
    var renderIconNode = function renderIconNode() {
      var prefixCls = props.prefixCls,
        progressDot = props.progressDot,
        stepNumber = props.stepNumber,
        status = props.status,
        title = props.title,
        description = props.description,
        icon = props.icon,
        iconPrefix = props.iconPrefix;
      if (slots.icon) {
        var _slots$icon;
        return _createVNode("span", {
          "class": "".concat(prefixCls, "-icon")
        }, [(_slots$icon = slots.icon) === null || _slots$icon === void 0 ? void 0 : _slots$icon.call(slots)]);
      }
      var iconNode;
      var iconClassName = classNames("".concat(prefixCls, "-icon"), "".concat(iconPrefix, "icon"), _defineProperty(_defineProperty(_defineProperty({}, "".concat(iconPrefix, "icon-").concat(icon), icon && isString(icon)), "".concat(iconPrefix, "icon-check"), !icon && status === 'finish'), "".concat(iconPrefix, "icon-cross"), !icon && status === 'error'));
      var iconStyle = {
        position: 'relative',
        left: '-1px'
      };
      var iconDot = _createVNode("span", {
        "class": "".concat(prefixCls, "-icon-dot")
      }, null);
      // `progressDot` enjoy the highest priority
      if (progressDot) {
        if (typeof progressDot === 'function') {
          iconNode = _createVNode("span", {
            "class": "".concat(prefixCls, "-icon")
          }, [progressDot(iconDot, {
            index: stepNumber - 1,
            status: status,
            title: title,
            description: description
          })]);
        } else {
          iconNode = _createVNode("span", {
            "class": "".concat(prefixCls, "-icon")
          }, [iconDot]);
        }
      } else if (icon && isString(icon)) {
        iconNode = _createVNode("span", {
          "class": "".concat(prefixCls, "-icon")
        }, [_createVNode(Icon, {
          "style": iconStyle,
          "size": iconSize.value,
          "type": icon
        }, null)]);
      } else if (isVNode(icon)) {
        iconNode = _createVNode("span", {
          "class": iconClassName
        }, [icon]);
      } else if (status === 'finish' || status === 'error') {
        iconNode = _createVNode("span", {
          "class": iconClassName
        }, null);
      } else {
        iconNode = _createVNode("span", {
          "class": "".concat(prefixCls, "-icon")
        }, [stepNumber]);
      }
      return iconNode;
    };
    var renderTitle = function renderTitle() {
      var _slots$title, _slots$title2;
      if (!(slots.title || props.title)) {
        return undefined;
      }
      return _createVNode("div", {
        "class": "".concat(props.prefixCls, "-item-title")
      }, [(_slots$title = (_slots$title2 = slots.title) === null || _slots$title2 === void 0 ? void 0 : _slots$title2.call(slots)) !== null && _slots$title !== void 0 ? _slots$title : props.title]);
    };
    return {
      renderIconNode: renderIconNode,
      renderTitle: renderTitle
    };
  },
  render: function render() {
    var _this$$slots$descript, _this$$slots, _this$$slots$descript2;
    var _this$$props = this.$props,
      prefixCls = _this$$props.prefixCls,
      itemWidth = _this$$props.itemWidth,
      _this$$props$status = _this$$props.status,
      status = _this$$props$status === void 0 ? 'wait' : _this$$props$status,
      icon = _this$$props.icon,
      adjustMarginRight = _this$$props.adjustMarginRight,
      description = _this$$props.description,
      title = _this$$props.title;
    var classString = classNames("".concat(prefixCls, "-item"), "".concat(prefixCls, "-item-").concat(status), this.$attrs.class, _defineProperty({}, "".concat(prefixCls, "-item-custom"), icon));
    var stepItemStyle = {};
    if (itemWidth) {
      stepItemStyle.width = itemWidth;
    }
    if (adjustMarginRight) {
      stepItemStyle.marginRight = adjustMarginRight;
    }
    return _createVNode("div", _objectSpread(_objectSpread({}, this.$attrs), {}, {
      "class": classString,
      "style": stepItemStyle
    }), [_createVNode("div", {
      "class": "".concat(prefixCls, "-item-tail")
    }, null), _createVNode("div", {
      "class": "".concat(prefixCls, "-item-icon")
    }, [this.renderIconNode()]), _createVNode("div", {
      "class": "".concat(prefixCls, "-item-content")
    }, [this.renderTitle(), (description || this.$slots.description) && _createVNode("div", {
      "class": "".concat(prefixCls, "-item-description")
    }, [(_this$$slots$descript = (_this$$slots = this.$slots) === null || _this$$slots === void 0 ? void 0 : (_this$$slots$descript2 = _this$$slots.description) === null || _this$$slots$descript2 === void 0 ? void 0 : _this$$slots$descript2.call(_this$$slots)) !== null && _this$$slots$descript !== void 0 ? _this$$slots$descript : description])])]);
  }
});