import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import { createVNode as _createVNode } from "vue";
import { unwrapFragment } from '../../utils/vue';
import { defineComponent, isVNode, provide, reactive, watch } from 'vue';
import Tabs from '../../tabs';
import Icon from "../../icon";
import Badge from "../../badge";
var tabClsPrefix = 'am-tab-bar-tab';
var TabBar = defineComponent({
  name: 'MTabBar',
  props: {
    prefixCls: {
      default: 'am-tab-bar'
    },
    className: {},
    hidden: {
      type: Boolean,
      default: false
    },
    placeholder: {
      type: String,
      default: '正在加载'
    },
    noRenderContent: {},
    prerenderingSiblingsNumber: {
      type: Number,
      default: 1
    },
    barTintColor: {
      type: String,
      default: 'white'
    },
    tintColor: {
      type: String,
      default: '#108ee9'
    },
    unselectedTintColor: {
      type: String,
      default: '#888'
    },
    tabBarPosition: {
      type: String,
      default: 'bottom'
    },
    animated: {
      type: Boolean,
      default: false
    },
    swipeable: {
      type: Boolean,
      default: false
    },
    value: {
      type: [Number, String]
    }
  },
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots;
    var store = reactive({
      currentTab: undefined
    });
    watch(function () {
      return props.value;
    }, function (value) {
      store.currentTab = value;
    }, {
      immediate: true
    });
    watch(function () {
      return store.currentTab;
    }, function (value) {
      emit('update:value', value);
    });
    var setCurrentTab = function setCurrentTab(tab) {
      var children = unwrapFragment(slots.default());
      store.currentTab = tab;
      emit('click', tab, children.find(function (it) {
        return it.key === tab;
      }));
    };
    var renderIcon = function renderIcon(tabProps) {
      var selected = tabProps.key === store.currentTab;
      var dot = tabProps.dot,
        badge = tabProps.badge,
        selectedIcon = tabProps.selectedIcon,
        icon = tabProps.icon,
        prefixCls = tabProps.prefixCls;
      var realIcon = selected ? selectedIcon !== null && selectedIcon !== void 0 ? selectedIcon : icon : icon;
      var iconDom = realIcon ? isVNode(realIcon) ? realIcon : _createVNode(Icon, {
        "class": "".concat(prefixCls, "-image"),
        "type": realIcon
      }, null) : null;
      if (badge) {
        return _createVNode(Badge, {
          "text": badge,
          "class": "".concat(prefixCls, "-badge tab-badge")
        }, {
          default: function _default() {
            return [' ', iconDom, ' '];
          }
        });
      }
      if (dot) {
        return _createVNode(Badge, {
          "dot": true,
          "class": "".concat(prefixCls, "-badge tab-dot")
        }, {
          default: function _default() {
            return [iconDom];
          }
        });
      }
      return iconDom;
    };
    var getContent = function getContent(tabProps) {
      var selected = tabProps.key === store.currentTab;
      var iconColor = selected ? tabProps.tintColor : tabProps.unselectedTintColor;
      return _createVNode("div", {
        "onClick": function onClick() {
          setCurrentTab(tabProps.key);
        },
        "class": _defineProperty(_defineProperty({}, tabClsPrefix, true), "".concat(tabClsPrefix, "-selected"), selected)
      }, [_createVNode("div", {
        "class": "".concat(tabClsPrefix, "-icon"),
        "style": {
          color: iconColor
        }
      }, [renderIcon(tabProps)]), _createVNode("p", {
        "class": "".concat(tabClsPrefix, "-title"),
        "style": {
          color: iconColor
        }
      }, [tabProps.title])]);
    };
    var getTabs = function getTabs() {
      var children = unwrapFragment(slots.default());
      return children.map(function (it, index) {
        var _it$key;
        return _objectSpread({
          key: (_it$key = it.key) !== null && _it$key !== void 0 ? _it$key : "tab_".concat(index)
        }, it.props);
      });
    };
    var renderTabBar = function renderTabBar() {
      var cls = "".concat(props.prefixCls, "-bar");
      if (props.hidden) {
        cls += " ".concat(props.prefixCls, "-bar-hidden-").concat(props.tabBarPosition);
      }
      var tabs = getTabs();
      return _createVNode("div", {
        "class": cls,
        "style": {
          backgroundColor: props.barTintColor
        }
      }, [tabs.map(function (it) {
        return getContent(it);
      })]);
    };
    provide('tabBar', {
      slots: slots,
      props: props
    });
    provide('tabBarStore', store);
    return {
      store: store,
      renderTabBar: renderTabBar,
      getTabs: getTabs
    };
  },
  render: function render() {
    var _this = this;
    var _this$$props = this.$props,
      prefixCls = _this$$props.prefixCls,
      animated = _this$$props.animated,
      swipeable = _this$$props.swipeable,
      prerenderingSiblingsNumber = _this$$props.prerenderingSiblingsNumber,
      tabBarPosition = _this$$props.tabBarPosition;
    var tabs = this.getTabs();
    var value = tabs.findIndex(function (it) {
      return it.key === _this.store.currentTab;
    });
    return _createVNode("div", {
      "class": prefixCls
    }, [_createVNode(Tabs, {
      "renderTabBar": this.renderTabBar,
      "value": value,
      "tabBarPosition": tabBarPosition,
      "animated": animated,
      "swipeable": swipeable,
      "prerenderingSiblingsNumber": prerenderingSiblingsNumber
    }, {
      default: function _default() {
        return [_this.$slots.default()];
      }
    })]);
  }
});
export default TabBar;