import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import { computed, defineComponent, onMounted, onUpdated, ref, watch } from 'vue';
import { unwrapFragment } from '../../utils/vue';
import Gesture from '../../vmc-gesture';
import { DIRECTION_DOWN, DIRECTION_LEFT, DIRECTION_RIGHT, DIRECTION_UP } from '../../vmc-gesture/config';
import DefaultTabBar from './default-tab-bar';
import TabPane from './tab-pane';
import { getTransformPropValue, setPxStyle, setTransform } from './utils';
import { camelize, flattenChildren, isValidElement } from "ant-design-vue/es/_util/props-util";
export var getPanDirection = function getPanDirection(direction) {
  switch (direction) {
    case DIRECTION_LEFT:
    case DIRECTION_RIGHT:
      return 'horizontal';
    case DIRECTION_UP:
    case DIRECTION_DOWN:
      return 'vertical';
    default:
      return 'none';
  }
};
var getTabTitle = function getTabTitle(it) {
  if (it.children && _typeof(it.children) === 'object' && it.children.title) {
    return it.children.title();
  }
  return it.props.title;
};
var tabsProps = function tabsProps() {
  return {
    /**
     * 使用卡片样式
     */
    card: {
      type: Boolean
    },
    /**
     * 激活的卡片背景色（未激活卡片的边框色与之相同）
     */
    activeCardColor: {
      type: String
    },
    /**
     * class前缀
     */
    prefixCls: {
      type: String,
      default: 'am-tabs'
    },
    useOnPan: {
      type: Boolean,
      default: true
    },
    renderTabBar: {
      type: Function
    },
    /** TabBar's position | default: top */
    tabBarPosition: {
      default: 'top'
    },
    /**
     * 当前激活的卡片的索引
     */
    value: {
      type: [Number],
      default: 0
    },
    /**
     * 是否支持手势
     */
    swipeable: {
      type: Boolean,
      default: true
    },
    /**
     * 与当前激活标签相邻的提前渲染的标签数量
     */
    prerenderingSiblingsNumber: {
      type: Number,
      default: 1
    },
    /**
     * 切换标签时是否有动画
     */
    animated: {
      type: Boolean,
      default: true
    },
    /**
     * 是否销毁未激活的标签页
     */
    destroyInactiveTab: {
      type: Boolean,
      default: false
    },
    /**
     * 切换卡片的滑动距离，0-1之间
     */
    distanceToChangeTab: {
      type: Number,
      default: 0.3
    },
    usePaged: {
      type: Boolean,
      default: true
    },
    /**
     * 标签页方向
     */
    tabDirection: {
      type: String,
      default: 'horizontal'
    },
    /** 标签下划线样式 */
    tabBarUnderlineStyle: {
      type: Object
    },
    /** 标签页背景颜色 */
    tabBarBackgroundColor: {
      type: String
    },
    page: {
      type: Number,
      default: 5
    },
    /** 激活的标签页文字颜色 */
    tabBarActiveTextColor: {
      type: String
    },
    /** 未激活的标签页文字颜色 */
    tabBarInactiveTextColor: {
      type: String
    },
    /** 标签栏文字样式 */
    tabBarTextStyle: {
      type: Object
    },
    /** use left instead of transform | default: false */
    useLeftInsteadTransform: {
      type: Boolean
    },
    onChange: Function
  };
};
var InternalTabs = defineComponent({
  DefaultTabBar: DefaultTabBar,
  name: 'MTabsInternal',
  props: _objectSpread(_objectSpread({}, tabsProps()), {}, {
    tabs: Array
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots;
    var contentPos = ref('');
    var isMoving = ref(false);
    var instanceId = ref(null);
    var currentTab = ref(0);
    var nextCurrentTab = ref(null);
    var prevCurrentTab = ref(null);
    watch(function () {
      return props.value;
    }, function (tab) {
      if (typeof tab === 'number') {
        currentTab.value = tab;
      }
    }, {
      immediate: true
    });
    var layoutRef = ref(null);
    watch(function () {
      return currentTab.value;
    }, function (index) {
      emit('update:value', index);
    });
    var onPan = computed(function () {
      var lastOffset = 0;
      var finalOffset = 0;
      var panDirection;
      var getLastOffset = function getLastOffset() {
        var isVertical = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : isTabVertical();
        var offset = +"".concat(lastOffset).replace('%', '');
        if ("".concat(lastOffset).indexOf('%') >= 0) {
          offset /= 100;
          offset *= isVertical ? layoutRef.value.clientHeight : layoutRef.value.clientWidth;
        }
        return offset;
      };
      return {
        onPanStart: function onPanStart(status) {
          if (!props.swipeable || !props.animated) {
            return;
          }
          panDirection = getPanDirection(status.direction);
          isMoving.value = true;
        },
        onPanMove: function onPanMove(status) {
          var swipeable = props.swipeable,
            animated = props.animated,
            useLeftInsteadTransform = props.useLeftInsteadTransform;
          if (!status.moveStatus || !layoutRef.value.value || !swipeable || !animated) {
            return;
          }
          var isVertical = isTabVertical();
          var offset = getLastOffset();
          if (isVertical) {
            offset += panDirection === 'horizontal' ? 0 : status.moveStatus.y;
          } else {
            offset += panDirection === 'vertical' ? 0 : status.moveStatus.x;
          }
          var canScrollOffset = isVertical ? -layoutRef.value.scrollHeight + layoutRef.value.clientHeight : -layoutRef.value.scrollWidth + layoutRef.value.clientWidth;
          offset = Math.min(offset, 0);
          offset = Math.max(offset, canScrollOffset);
          setPxStyle(layoutRef.value.value, offset, 'px', isVertical, useLeftInsteadTransform);
          finalOffset = offset;
        },
        onPanEnd: function onPanEnd() {
          if (!props.swipeable || !props.animated) {
            return;
          }
          lastOffset = finalOffset;
          var isVertical = isTabVertical();
          var offsetIndex = getOffsetIndex(finalOffset, isVertical ? layoutRef.value.clientHeight : layoutRef.value.clientWidth);
          isMoving.value = false;
          if (offsetIndex === currentTab.value) {
            if (props.usePaged) {
              setTransform(layoutRef.value.style, getContentPosByIndex(offsetIndex, isTabVertical(), props.useLeftInsteadTransform));
            }
          } else {
            goToTab(offsetIndex);
          }
        },
        setCurrentOffset: function setCurrentOffset(offset) {
          return lastOffset = offset;
        }
      };
    });
    /** on tab click */
    var onTabClick = function onTabClick(tab, index) {
      emit('tab-click', index);
    };
    var isTabVertical = function isTabVertical() {
      var direction = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : props.tabDirection;
      return direction === 'vertical';
    };
    onMounted(function () {
      nextCurrentTab.value = currentTab.value;
      instanceId.value = instanceId.value++;
      contentPos.value = getContentPosByIndex(currentTab.value, isTabVertical(props.tabDirection), props.useLeftInsteadTransform);
      prevCurrentTab.value = currentTab.value;
    });
    var getOffsetIndex = function getOffsetIndex(current, width) {
      var threshold = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : props.distanceToChangeTab || 0;
      var ratio = Math.abs(current / width);
      var direction = ratio > currentTab.value ? '<' : '>';
      var index = Math.floor(ratio);
      switch (direction) {
        case '<':
          return ratio - index > threshold ? index + 1 : index;
        case '>':
          return 1 - ratio + index > threshold ? index : index + 1;
        default:
          return Math.round(ratio);
      }
    };
    var baseGoToTab = function baseGoToTab(index) {
      var force = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var setState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      if (!force && nextCurrentTab.value === index) {
        return false;
      }
      nextCurrentTab.value = index;
      if (index >= 0 && index < props.tabs.length) {
        if (!force) {
          emit('change', props.tabs[index], index);
        }
        currentTab.value = index;
        if (setState) {
          setState();
        }
      }
      return true;
    };
    var getTabBarBaseProps = function getTabBarBaseProps() {
      var animated = props.animated,
        tabBarActiveTextColor = props.tabBarActiveTextColor,
        tabBarBackgroundColor = props.tabBarBackgroundColor,
        tabBarInactiveTextColor = props.tabBarInactiveTextColor,
        tabBarPosition = props.tabBarPosition,
        tabBarTextStyle = props.tabBarTextStyle,
        tabBarUnderlineStyle = props.tabBarUnderlineStyle;
      return {
        activeTab: currentTab.value,
        animated: animated,
        card: props.card,
        activeCardColor: props.activeCardColor,
        page: props.page,
        goToTab: tabClickGoToTab,
        tabBarActiveTextColor: tabBarActiveTextColor,
        tabBarBackgroundColor: tabBarBackgroundColor,
        tabBarInactiveTextColor: tabBarInactiveTextColor,
        tabBarPosition: tabBarPosition,
        tabBarTextStyle: tabBarTextStyle,
        tabBarUnderlineStyle: tabBarUnderlineStyle,
        instanceId: instanceId.value
      };
    };
    var getSubElements = function getSubElements() {
      var children = unwrapFragment(slots.default());
      var subElements = {};
      return function () {
        var defaultPrefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '$i$-';
        var allPrefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '$ALL$';
        if (Array.isArray(children)) {
          children.forEach(function (child, index) {
            if (child.key) {
              subElements[child.key] = child;
            }
            subElements["".concat(defaultPrefix).concat(index)] = child;
          });
        } else if (children) {
          subElements[allPrefix] = children;
        }
        return subElements;
      };
    };
    var goToTab = function goToTab(index) {
      var force = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var usePaged = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : props.usePaged;
      var tabDirection = props.tabDirection,
        useLeftInsteadTransform = props.useLeftInsteadTransform;
      var setState = function setState() {};
      if (usePaged) {
        setState = function setState() {
          contentPos.value = getContentPosByIndex(index, isTabVertical(tabDirection), useLeftInsteadTransform);
        };
      }
      return baseGoToTab(index, force, setState);
    };
    var tabClickGoToTab = function tabClickGoToTab(index) {
      goToTab(index, false, true);
    };
    watch(function () {
      return props.value;
    }, function (value) {
      tabClickGoToTab(value);
    });
    var getContentPosByIndex = function getContentPosByIndex(index, isVertical) {
      var useLeft = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
      var value = "".concat(-index * 100, "%");
      onPan.value.setCurrentOffset(value);
      if (useLeft) {
        return "".concat(value);
      } else {
        var translate = isVertical ? "0px, ".concat(value) : "".concat(value, ", 0px");
        // fix: content overlay TabBar on iOS 10. ( 0px -> 1px )
        return "translate3d(".concat(translate, ", 1px)");
      }
    };
    var onSwipe = function onSwipe(status) {
      var tabBarPosition = props.tabBarPosition,
        swipeable = props.swipeable,
        usePaged = props.usePaged;
      if (!swipeable || !usePaged || isTabVertical()) {
        return;
      }
      switch (tabBarPosition) {
        case 'top':
        case 'bottom':
          switch (status.direction) {
            case DIRECTION_LEFT:
              if (!isTabVertical()) {
                goToTab(prevCurrentTab.value + 1);
              }
              break;
            case DIRECTION_UP:
              if (isTabVertical()) {
                goToTab(prevCurrentTab.value + 1);
              }
              break;
            case DIRECTION_RIGHT:
              if (!isTabVertical()) {
                goToTab(prevCurrentTab.value - 1);
              }
              break;
            case DIRECTION_DOWN:
              if (isTabVertical()) {
                goToTab(prevCurrentTab.value - 1);
              }
              break;
          }
          break;
      }
    };
    var getContentStyle = function getContentStyle() {
      var animated = props.animated,
        useLeftInsteadTransform = props.useLeftInsteadTransform;
      if (animated) {
        if (useLeftInsteadTransform) {
          return _objectSpread({
            position: 'relative'
          }, isTabVertical() ? {
            top: contentPos.value
          } : {
            left: contentPos.value
          });
        }
        return getTransformPropValue(contentPos.value);
      }
      return _objectSpread({
        position: 'relative'
      }, isTabVertical() ? {
        top: "".concat(-currentTab.value * 100, "%")
      } : {
        left: "".concat(-currentTab.value * 100, "%")
      });
    };
    var renderContent = function renderContent(children) {
      var prefixCls = props.prefixCls,
        animated = props.animated,
        destroyInactiveTab = props.destroyInactiveTab,
        useLeftInsteadTransform = props.useLeftInsteadTransform;
      var contentCls = "".concat(prefixCls, "-content-wrap");
      if (animated && !isMoving.value) {
        contentCls += " ".concat(contentCls, "-animated");
      }
      var contentStyle = getContentStyle();
      var _getTabBarBaseProps = getTabBarBaseProps(),
        instanceId = _getTabBarBaseProps.instanceId;
      return _createVNode("div", {
        "class": contentCls,
        "style": contentStyle,
        "ref": function ref(el) {
          layoutRef.value = el;
        }
      }, [children.map(function (tab, index) {
        var cls = "".concat(prefixCls, "-pane-wrap");
        if (currentTab.value === index) {
          cls += " ".concat(cls, "-active");
        } else {
          cls += " ".concat(cls, "-inactive");
        }
        return _createVNode(TabPane, {
          "key": tab.key,
          "class": cls,
          "forceRender": tab.props.forceRender,
          "title": getTabTitle(tab),
          "active": currentTab.value === index,
          "role": "tabpanel",
          "aria-hidden": currentTab.value !== index,
          "aria-labelledby": "m-tabs-".concat(instanceId, "-").concat(index),
          "fixX": isTabVertical(),
          "fixY": !isTabVertical()
        }, {
          default: function _default() {
            return [tab];
          }
        });
      })]);
    };
    onUpdated(function () {
      prevCurrentTab.value = currentTab.value;
    });
    return {
      isTabVertical: isTabVertical,
      getTabBarBaseProps: getTabBarBaseProps,
      onPan: onPan,
      onTabClick: onTabClick,
      onSwipe: onSwipe,
      currentTab: currentTab,
      renderContent: renderContent
    };
  },
  render: function render() {
    var _this$$slots,
      _this$$slots$default,
      _this$renderTabBar,
      _this$renderTabBar2,
      _this = this;
    var prefixCls = this.prefixCls,
      tabBarPosition = this.tabBarPosition,
      tabDirection = this.tabDirection,
      useOnPan = this.useOnPan,
      tabs = this.tabs;
    var isTabVertical = this.isTabVertical(tabDirection);
    var children = unwrapFragment((_this$$slots = this.$slots) === null || _this$$slots === void 0 ? void 0 : (_this$$slots$default = _this$$slots.default) === null || _this$$slots$default === void 0 ? void 0 : _this$$slots$default.call(_this$$slots)) || [];
    var tabBarProps = _objectSpread(_objectSpread({}, this.getTabBarBaseProps()), {}, {
      tabs: tabs
    });
    var onPan = !isTabVertical && useOnPan ? this.onPan : {};
    var content = [_createVNode("div", {
      "key": "tabBar",
      "class": "".concat(prefixCls, "-tab-bar-wrap")
    }, [(_this$renderTabBar = (_this$renderTabBar2 = this.renderTabBar) === null || _this$renderTabBar2 === void 0 ? void 0 : _this$renderTabBar2.call(this, tabBarProps)) !== null && _this$renderTabBar !== void 0 ? _this$renderTabBar : _createVNode(DefaultTabBar, _objectSpread(_objectSpread({}, tabBarProps), {}, {
      "onTabClick": function onTabClick(tab, index) {
        _this.onTabClick(tab, index);
      }
    }), null)]), _createVNode(Gesture, {
      "key": "$content",
      "onSwipe": this.onSwipe,
      onPan: onPan
    }, {
      default: function _default() {
        return [_this.renderContent(children)];
      }
    })];
    return _createVNode("div", {
      "class": "".concat(prefixCls, " ").concat(prefixCls, "-").concat(tabDirection, " ").concat(prefixCls, "-").concat(tabBarPosition)
    }, [tabBarPosition === 'top' || tabBarPosition === 'left' ? content : content.reverse()]);
  }
});
export default defineComponent({
  name: 'MTabs',
  props: _objectSpread({}, tabsProps()),
  setup: function setup(props, _ref2) {
    var attrs = _ref2.attrs,
      slots = _ref2.slots;
    var parseTabList = function parseTabList(children) {
      return children.map(function (node) {
        if (isValidElement(node)) {
          var _props = _objectSpread({}, node.props || {});
          for (var _i = 0, _Object$entries = Object.entries(_props); _i < _Object$entries.length; _i++) {
            var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
              k = _Object$entries$_i[0],
              v = _Object$entries$_i[1];
            delete _props[k];
            _props[camelize(k)] = v;
          }
          var _slots = node.children || {};
          var key = node.key !== undefined ? node.key : undefined;
          var _props$tab = _props.tab,
            tab = _props$tab === void 0 ? _slots.tab : _props$tab,
            disabled = _props.disabled,
            forceRender = _props.forceRender,
            closable = _props.closable,
            animated = _props.animated,
            active = _props.active,
            title = _props.title,
            destroyInactiveTabPane = _props.destroyInactiveTabPane;
          return _objectSpread(_objectSpread({
            key: key
          }, _props), {}, {
            node: node,
            closeIcon: _slots.closeIcon,
            title: title,
            tab: tab,
            disabled: disabled === '' || disabled,
            forceRender: forceRender === '' || forceRender,
            closable: closable === '' || closable,
            animated: animated === '' || animated,
            active: active === '' || active,
            destroyInactiveTabPane: destroyInactiveTabPane === '' || destroyInactiveTabPane
          });
        }
        return null;
      }).filter(function (tab) {
        return tab;
      });
    };
    return function () {
      var _slots$default;
      var tabs = parseTabList(flattenChildren((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots)));
      return _createVNode(InternalTabs, _objectSpread(_objectSpread(_objectSpread({}, props), attrs), {}, {
        "tabs": tabs
      }), slots);
    };
  }
});