import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent, onBeforeUpdate, ref } from 'vue';
import { filterHTMLAttrs } from '../../utils/dom';
import { getPxStyle, getTransformPropValue } from './utils';
export default defineComponent({
  inheritAttrs: false,
  name: 'TabPane',
  props: {
    role: {
      type: String
    },
    title: {
      type: [String, Object]
    },
    active: {
      type: Boolean
    },
    fixX: {
      type: Boolean,
      default: true
    },
    fixY: {
      type: Boolean,
      default: true
    },
    forceRender: Boolean
  },
  setup: function setup(props) {
    var layout = ref(null);
    var offsetX = ref(0);
    var offsetY = ref(0);
    var setLayout = function setLayout(div) {
      layout.value = div;
    };
    onBeforeUpdate(function () {
      if (props.active) {
        offsetX.value = 0;
        offsetY.value = 0;
      } else {
        offsetX.value = layout.value.scrollLeft;
        offsetY.value = layout.value.scrollTop;
      }
    });
    return {
      setLayout: setLayout,
      offsetX: offsetX,
      offsetY: offsetY
    };
  },
  render: function render() {
    var _this$$props = this.$props,
      fixX = _this$$props.fixX,
      fixY = _this$$props.fixY;
    var style = _objectSpread(_objectSpread({}, fixX && this.offsetX ? getTransformPropValue(getPxStyle(-this.offsetX, 'px', false)) : {}), fixY && this.offsetY ? getTransformPropValue(getPxStyle(-this.offsetY, 'px', true)) : {});
    return _createVNode("div", _objectSpread(_objectSpread({}, filterHTMLAttrs(this.$attrs)), {}, {
      "style": style,
      "ref": this.setLayout
    }), [this.active || this.forceRender ? this.$slots.default() : undefined]);
  }
});