import { createVNode as _createVNode, createTextVNode as _createTextVNode } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import classnames from 'classnames';
import { defineComponent, onBeforeUnmount, onMounted, onUpdated, reactive, ref, watch } from 'vue';
import { renderLabel } from '../../input/src/utils';
import List from '../../list';
import { creatFormComponentProps, useFormComponent } from '../../mixins/form-component';
import { IS_IOS } from '../../utils/exenv';
import TouchFeedback from '../../vmc-feedback';
function fixControlledValue(value) {
  if (typeof value === 'undefined' || value === null) {
    return '';
  }
  return value;
}
var regexAstralSymbols = /[\uD800-\uDBFF][\uDC00-\uDFFF]|\n/g;
function countSymbols() {
  var text = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return text.replace(regexAstralSymbols, '_').length;
}
export default defineComponent({
  name: 'MTextarea',
  props: _objectSpread(_objectSpread({}, creatFormComponentProps()), {}, {
    prefixCls: {
      type: String,
      default: 'am-textarea'
    },
    prefixListCls: {
      type: String,
      default: 'am-list'
    },
    title: {
      type: [String, Object]
    },
    maxLength: {
      type: Number
    },
    name: {
      type: String
    },
    placeholder: {
      default: ''
    },
    clearable: {
      type: Boolean,
      default: false
    },
    rows: {
      type: Number,
      default: 1
    },
    count: {
      type: Number
    },
    autoHeight: {
      type: Boolean,
      default: true
    },
    labelNumber: {
      type: Number,
      default: 5
    }
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots,
      attrs = _ref.attrs;
    var _useFormComponent = useFormComponent(props, {
        emit: emit
      }),
      isReadonly = _useFormComponent.isReadonly,
      isDisabled = _useFormComponent.isDisabled;
    var debounceTimeout = ref(null);
    var state = reactive({
      focus: false,
      value: props.value || ''
    });
    var textareaRef = ref(null);
    var reAlignHeight = function reAlignHeight() {
      if (props.autoHeight) {
        var textareaDom = textareaRef.value;
        textareaDom.style.height = ''; // 字数减少时能自动减小高度
        textareaDom.style.height = "".concat(textareaDom.scrollHeight + 2, "px");
      }
    };
    watch(function () {
      return props.value;
    }, function (value) {
      state.value = fixControlledValue(value);
      reAlignHeight();
    });
    watch(function () {
      return state.value;
    }, function (value) {
      emit('update:value', value);
    });
    var focus = function focus() {
      textareaRef.value.focus();
    };
    var onChange = function onChange(e) {
      var value = e.target.value;
      state.value = value;
      emit('change', value);
    };
    var onBlur = function onBlur(e) {
      debounceTimeout.value = setTimeout(function () {
        if (document.activeElement !== textareaRef.value) {
          state.focus = false;
        }
      }, 150);
      var value = e.currentTarget.value;
      // fix autoFocus item blur with flash
      setTimeout(function () {
        // fix ios12 wechat browser click failure after input
        if (document.body) {
          document.body.scrollTop = document.body.scrollTop;
        }
      }, 100);
      emit('blur', value);
    };
    var onFocus = function onFocus(e) {
      if (debounceTimeout.value) {
        clearTimeout(debounceTimeout.value);
        debounceTimeout.value = null;
      }
      state.focus = true;
      var value = e.currentTarget.value;
      emit('focus', value);
    };
    var clearInput = function clearInput() {
      state.value = '';
      emit('change', '');
    };
    var onInput = function onInput(e) {
      state.value = e.target.value;
      if (state.focus) {
        reAlignHeight();
      }
    };
    onMounted(function () {
      reAlignHeight();
    });
    onUpdated(function () {
      if (state.focus) {
        reAlignHeight();
      }
    });
    onBeforeUnmount(function () {
      if (debounceTimeout.value) {
        clearTimeout(debounceTimeout.value);
        debounceTimeout.value = null;
      }
    });
    return {
      textareaRef: textareaRef,
      state: state,
      isDisabled: isDisabled,
      isReadonly: isReadonly,
      onInput: onInput,
      onChange: onChange,
      onBlur: onBlur,
      onFocus: onFocus,
      clearInput: clearInput,
      focus: focus
    };
  },
  render: function render() {
    var _this = this;
    var _this$$props = this.$props,
      prefixCls = _this$$props.prefixCls,
      prefixListCls = _this$$props.prefixListCls,
      editable = _this$$props.editable,
      clearable = _this$$props.clearable,
      count = _this$$props.count,
      labelNumber = _this$$props.labelNumber,
      title = _this$$props.title,
      autoHeight = _this$$props.autoHeight,
      disabled = _this$$props.disabled;
    var _ref2 = this.state,
      value = _ref2.value,
      focus = _ref2.focus;
    var hasCount = count > 0 && this.rows > 1;
    var wrapCls = classnames("".concat(prefixListCls, "-item"), "".concat(prefixCls, "-item"), _defineProperty(_defineProperty(_defineProperty(_defineProperty({}, "".concat(prefixCls, "-disabled"), disabled), "".concat(prefixCls, "-item-single-line"), this.rows === 1 && !autoHeight), "".concat(prefixCls, "-focus"), focus), "".concat(prefixCls, "-has-count"), hasCount));
    var characterLength = countSymbols(value);
    var lengthCtrlProps = {};
    if (count > 0) {
      // Note: If in the iOS environment of dev-tools, It will fail.
      if (IS_IOS) {
        var entValue = value ? value.replace(regexAstralSymbols, '_') : '';
        var entLen = entValue ? entValue.split('_').length - 1 : 0;
        lengthCtrlProps.maxLength = count + entLen - characterLength + (value ? value.length : 0);
      } else {
        lengthCtrlProps.maxLength = count - characterLength + (value ? value.length : 0);
      }
    }
    var slots = {
      control: function control() {
        return _createVNode("div", {
          "class": "".concat(prefixCls, "-control")
        }, [_createVNode("textarea", _objectSpread(_objectSpread({
          "ref": "textareaRef"
        }, lengthCtrlProps), {}, {
          "rows": _this.rows,
          "disabled": _this.isDisabled,
          "name": _this.name,
          "placeholder": _this.placeholder,
          "value": value,
          "oninput": _this.onInput,
          "onChange": _this.onChange,
          "onBlur": _this.onBlur,
          "onFocus": _this.onFocus,
          "readOnly": !editable
        }), null), clearable && editable && value && characterLength > 0 && // @ts-ignore
        _createVNode(TouchFeedback, {
          "activeClassName": "".concat(prefixCls, "-clear-active")
        }, {
          default: function _default() {
            return [_createVNode("div", {
              "class": "".concat(prefixCls, "-clear"),
              "onClick": _this.clearInput
            }, null)];
          }
        }), hasCount && _createVNode("span", {
          "class": "".concat(prefixCls, "-count")
        }, [_createVNode("span", null, [value ? characterLength : 0]), _createTextVNode("/"), count])]);
      }
    };
    return _createVNode(List.Item, {
      "class": wrapCls,
      "required": this.required,
      "disabled": this.isDisabled,
      "error": this.error,
      "errorMessage": this.errorMessage,
      "errorDisplayType": this.errorDisplayType,
      "title": renderLabel(this.$props, this.$slots)
    }, slots);
  }
});