import { createApp, Transition, createVNode as _createVNode } from 'vue';
import Icon from '../../icon';
import { v4 } from 'uuid';
import { isNull } from "../../utils/util";
var prefixCls = 'am-toast';
var iconTypes = {
  info: '',
  success: 'success',
  fail: 'fail',
  offline: 'dislike',
  loading: 'loading'
};
var DEFAULT_CONFIG = {
  duration: 3,
  mask: false,
  position: 'center'
};
var map = new Map();
var destroyAll = function destroyAll() {
  map.forEach(function (value, key) {
    value.app.unmount();
    value.container.remove();
    map.delete(key);
  });
};
var destroy = function destroy(key) {
  var record = map.get(key);
  if (!record) {
    return;
  }
  record.app.unmount();
  record.container.remove();
  map.delete(key);
};
var offset = {
  top: 20,
  center: 50,
  bottom: 80
};
function notice(props) {
  Object.keys(DEFAULT_CONFIG).forEach(function (key) {
    if (isNull(props[key])) {
      props[key] = DEFAULT_CONFIG[key];
    }
  });
  var content = props.content,
    type = props.type,
    duration = props.duration,
    onClose = props.onClose,
    mask = props.mask,
    position = props.position;
  var key = v4();
  var div = document.createElement('div');
  destroyAll();
  var options = {
    key: key,
    placement: 'top',
    prefixCls: prefixCls,
    message: content,
    icon: iconTypes[type] ? _createVNode(Icon, {
      "type": iconTypes[type],
      "size": 'lg'
    }, null) : undefined,
    duration: duration * 1000,
    onClose: onClose
  };
  document.body.appendChild(div);
  var app = createApp({
    render: function render() {
      var _offset$position;
      return _createVNode(Transition, null, {
        default: function _default() {
          return [_createVNode("div", {
            "key": 'fade-in'
          }, [_createVNode("div", {
            "class": ['am-mask', 'am-toast-mask'],
            "style": {
              pointerEvents: mask ? 'all' : 'none'
            }
          }, [_createVNode("div", {
            "class": "am-mask-content"
          }, [_createVNode("div", {
            "class": "am-toast-wrap"
          }, [_createVNode("div", {
            "class": "am-toast-main am-toast-main-icon",
            "style": {
              top: "".concat((_offset$position = offset[position]) !== null && _offset$position !== void 0 ? _offset$position : 50, "%")
            }
          }, [_createVNode("div", {
            "class": "am-toast-icon"
          }, [options.icon ? _createVNode("span", {
            "class": "".concat(prefixCls, "-notice-icon")
          }, [options.icon]) : undefined]), _createVNode("div", {
            "class": "am-auto-center"
          }, [_createVNode("div", {
            "class": "am-auto-center-content"
          }, [_createVNode("span", null, [content])])])])])])])])];
        }
      });
    }
  });
  app.mount(div);
  map.set(key, {
    container: div,
    app: app
  });
  setTimeout(function () {
    destroy(key);
  }, duration * 1000);
  return {
    hide: function hide() {
      destroy(key);
    }
  };
}
export default {
  install: function install() {},
  show: function show(props) {
    return notice(props);
  },
  info: function info(content, duration, onClose) {
    var mask = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    return notice({
      content: content,
      type: 'info',
      duration: duration,
      onClose: onClose,
      mask: mask
    });
  },
  success: function success(content, duration, onClose) {
    var mask = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    return notice({
      content: content,
      type: 'success',
      duration: duration,
      onClose: onClose,
      mask: mask
    });
  },
  fail: function fail(content, duration, onClose) {
    var mask = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    return notice({
      content: content,
      type: 'fail',
      duration: duration,
      onClose: onClose,
      mask: mask
    });
  },
  offline: function offline(content, duration, onClose) {
    var mask = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    return notice({
      content: content,
      type: 'offline',
      duration: duration,
      onClose: onClose,
      mask: mask
    });
  },
  loading: function loading(content, duration, onClose, mask) {
    return notice({
      content: content,
      type: 'loading',
      duration: duration,
      onClose: onClose,
      mask: mask
    });
  },
  config: function config(props) {
    Object.keys(props).forEach(function (key) {
      DEFAULT_CONFIG[key] = props[key];
    });
  }
};