import _extends from "@babel/runtime/helpers/esm/extends";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import { ref, watch } from 'vue';
import isequal from 'lodash.isequal';
import cloneDeep from 'lodash.clonedeep';
var isNullOrUndefined = function isNullOrUndefined(value) {
  return value === undefined || value === null;
};
var basicTypes = ['number', 'boolean', 'string'];
function isEqual(a, b) {
  if (isNullOrUndefined(a) && isNullOrUndefined(b)) {
    return true;
  } else if (isNullOrUndefined(a) || isNullOrUndefined(b)) {
    return false;
  }
  var compareA = basicTypes.includes(_typeof(a)) ? a.toString() : a;
  var compareB = basicTypes.includes(_typeof(b)) ? b.toString() : b;
  return compareA === compareB;
}
var defaultOptions = {
  prop: 'value',
  defaultValue: undefined,
  autoEmit: true
};
export var useLocalValue = function useLocalValue(props, emit) {
  var _props$options$prop;
  var nameOrOptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'value';
  var defaultValue = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : undefined;
  var options = _typeof(nameOrOptions) === 'object' ? _extends({}, defaultOptions, nameOrOptions) : _extends({}, defaultOptions, {
    prop: nameOrOptions,
    defaultValue: defaultValue
  });
  var localValue = ref((_props$options$prop = props[options.prop]) !== null && _props$options$prop !== void 0 ? _props$options$prop : options.defaultValue);
  watch(function () {
    return props[options.prop];
  }, function (value) {
    if (!isequal(value, localValue.value)) {
      if (value === null || value === undefined) {
        localValue.value = options.defaultValue;
      } else {
        if (_typeof(value) === 'object') {
          localValue.value = cloneDeep(value);
        } else {
          localValue.value = value;
        }
      }
    }
  }, {
    deep: true
  });
  if (options.autoEmit) {
    watch(function () {
      return localValue.value;
    }, function (value) {
      var finalValue = options.onBeforeEmit ? options.onBeforeEmit(value) : value;
      if (options.prop) {
        emit("update:".concat(options.prop), finalValue);
      } else {
        emit('update', finalValue);
      }
    }, {
      deep: true
    });
  }
  return {
    localValue: localValue
  };
};