import { createVNode as _createVNode } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _typeof from "@babel/runtime/helpers/esm/typeof";
import { getPropsData } from '../../utils/props-util';
import BaseMixin from 'ant-design-vue/es/_util/BaseMixin';
import { isEmptyElement, initDefaultProps } from 'ant-design-vue/es/_util/props-util';
import { cloneElement } from 'ant-design-vue/es/_util/vnode';
import { unwrapFragment } from '../../utils/vue';
import openAnimationFactory from './open-animation-factory';
import { collapseProps } from './common-props';
import { getDataAndAriaProps } from 'ant-design-vue/es/_util/util';
import { defineComponent, ref, watch } from 'vue';
function _toArray(activeKey) {
  var currentActiveKey = activeKey;
  if (!Array.isArray(currentActiveKey)) {
    var activeKeyType = _typeof(currentActiveKey);
    currentActiveKey = activeKeyType === 'number' || activeKeyType === 'string' ? [currentActiveKey] : [];
  }
  return currentActiveKey.map(function (key) {
    return String(key);
  });
}
export default defineComponent({
  name: 'Collapse',
  mixins: [BaseMixin],
  inheritAttrs: false,
  props: initDefaultProps(collapseProps(), {
    prefixCls: 'rc-collapse',
    accordion: false,
    destroyInactivePanel: false
  }),
  setup: function setup(props, _ref) {
    var emit = _ref.emit,
      slots = _ref.slots;
    var activeKey = props.activeKey,
      defaultActiveKey = props.defaultActiveKey,
      openAnimation = props.openAnimation,
      prefixCls = props.prefixCls;
    var currentActiveKey = defaultActiveKey;
    if (props.activeKey !== undefined) {
      currentActiveKey = activeKey;
    }
    var currentOpenAnimations = ref(openAnimation || openAnimationFactory(prefixCls));
    var stateActiveKey = ref(_toArray(currentActiveKey));
    watch(function () {
      return props.activeKey;
    }, function (val) {
      stateActiveKey.value = _toArray(val);
    });
    watch(function () {
      return props.openAnimation;
    }, function (val) {
      currentOpenAnimations.value = val;
    });
    var setActiveKey = function setActiveKey(activeKey) {
      if (props.activeKey === undefined) {
        stateActiveKey.value = activeKey;
      }
      emit('change', props.accordion ? activeKey[0] : activeKey);
    };
    var onClickItem = function onClickItem(key) {
      var activeKey = stateActiveKey.value;
      if (props.accordion) {
        activeKey = activeKey[0] === key ? [] : [key];
      } else {
        activeKey = _toConsumableArray(activeKey);
        var index = activeKey.indexOf(key);
        var isActive = index > -1;
        if (isActive) {
          // remove active state
          activeKey.splice(index, 1);
        } else {
          activeKey.push(key);
        }
      }
      setActiveKey(activeKey);
    };
    var getNewChild = function getNewChild(child, index) {
      var _child$key;
      if (isEmptyElement(child)) {
        return;
      }
      var activeKey = stateActiveKey.value;
      var prefixCls = props.prefixCls,
        accordion = props.accordion,
        destroyInactivePanel = props.destroyInactivePanel,
        expandIcon = props.expandIcon;

      // If there is no key provide, use the panel order as default key
      var key = String((_child$key = child.key) !== null && _child$key !== void 0 ? _child$key : index);
      var _ref2 = getPropsData(child),
        header = _ref2.header,
        headerClass = _ref2.headerClass,
        disabled = _ref2.disabled;
      var isActive;
      if (accordion) {
        isActive = activeKey[0] === key;
      } else {
        isActive = activeKey.indexOf(key) > -1;
      }
      var panelEvents = {};
      if (!disabled && disabled !== '') {
        panelEvents = {
          onItemClick: onClickItem
        };
      }
      return cloneElement(child, _objectSpread({
        key: key,
        panelKey: key,
        header: header,
        headerClass: headerClass,
        isActive: isActive,
        prefixCls: prefixCls,
        destroyInactivePanel: destroyInactivePanel,
        openAnimation: currentOpenAnimations.value,
        accordion: accordion,
        expandIcon: expandIcon
      }, panelEvents));
    };
    var getItems = function getItems() {
      var _slots$default;
      var newChildren = [];
      var children = unwrapFragment((_slots$default = slots.default) === null || _slots$default === void 0 ? void 0 : _slots$default.call(slots));
      children && children.forEach(function (child, index) {
        newChildren.push(getNewChild(child, index));
      });
      return newChildren;
    };
    return {
      currentOpenAnimations: currentOpenAnimations,
      stateActiveKey: stateActiveKey,
      getItems: getItems,
      onClickItem: onClickItem,
      getNewChild: getNewChild
    };
  },
  render: function render() {
    var _this$$props = this.$props,
      prefixCls = _this$$props.prefixCls,
      accordion = _this$$props.accordion;
    var _this$$attrs = this.$attrs,
      className = _this$$attrs.class,
      style = _this$$attrs.style;
    var collapseClassName = _defineProperty(_defineProperty({}, prefixCls, true), className, className);
    return _createVNode("div", _objectSpread(_objectSpread({
      "class": collapseClassName
    }, getDataAndAriaProps(this.$attrs)), {}, {
      "style": style,
      "role": accordion ? 'tablist' : null
    }), [this.getItems()]);
  }
});