import { createVNode as _createVNode } from "vue";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent } from 'vue';
var PickerItem = defineComponent({
  name: 'PickerItem',
  props: {
    value: {},
    label: {}
  }
});
export default function PickerMixin(ComposedComponent) {
  return defineComponent({
    name: 'PickerMixin',
    props: {
      data: {
        type: Array,
        required: true
      },
      value: {},
      disabled: {
        type: Boolean,
        default: false
      },
      itemStyle: {},
      prefixCls: {},
      indicatorStyle: {},
      indicatorClassName: {},
      defaultValue: {},
      noAnimate: {}
    },
    setup: function setup(props, _ref) {
      var slots = _ref.slots;
      var select = function select(value, itemHeight, scrollTo) {
        for (var i = 0, len = props.data.length; i < len; i++) {
          if (props.data[i].value === value) {
            selectByIndex(i, itemHeight, scrollTo);
            return;
          }
        }
        selectByIndex(0, itemHeight, scrollTo);
      };
      var selectByIndex = function selectByIndex(index, itemHeight, zscrollTo) {
        if (index < 0 || index >= props.data.length || !itemHeight) {
          return;
        }
        zscrollTo(index * itemHeight);
      };
      var computeChildIndex = function computeChildIndex(top, itemHeight, childrenLength) {
        var index = Math.round(top / itemHeight);
        return Math.min(index, childrenLength - 1);
      };
      var doScrollingComplete = function doScrollingComplete(top, itemHeight, fireValueChange) {
        var index = computeChildIndex(top, itemHeight, props.data.length);
        var child = props.data[index];
        if (child) {
          fireValueChange(child.value);
        } else if (console.warn) {
          console.warn('child not found', props.data, index);
        }
      };
      return {
        doScrollingComplete: doScrollingComplete,
        computeChildIndex: computeChildIndex,
        select: select
      };
    },
    render: function render() {
      var _this$$slots$default,
        _this$$slots,
        _this = this;
      return _createVNode(ComposedComponent, _objectSpread(_objectSpread(_objectSpread({}, this.$props), this.$attrs), {}, {
        doScrollingComplete: this.doScrollingComplete,
        computeChildIndex: this.computeChildIndex,
        select: this.select
      }), {
        default: function _default() {
          return [(_this$$slots$default = (_this$$slots = _this.$slots).default) === null || _this$$slots$default === void 0 ? void 0 : _this$$slots$default.call(_this$$slots)];
        }
      });
    }
  });
}