import { Fragment as _Fragment, createVNode as _createVNode } from "vue";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import { defineComponent, reactive, ref, watch } from 'vue';
import { cloneVNodes, setListeners, setProps } from '../utils/vnode';
import { unwrapFragment } from '../utils/vue';
import { PopupPickerProps } from './popup-picker-types';
export default function PopupMixin(getModal, newProps) {
  return defineComponent({
    name: 'PopupMixin',
    inheritAttrs: false,
    props: _objectSpread({}, PopupPickerProps),
    setup: function setup(props, _ref) {
      var emit = _ref.emit,
        slots = _ref.slots;
      var picker = ref(null);
      var state = reactive({
        pickerValue: props.value !== undefined ? props.value : null,
        open: props.open || false
      });
      watch(function () {
        return state.open;
      }, function (open) {
        emit('open-change', open);
      });
      watch(function () {
        return props.value;
      }, function (value) {
        state.pickerValue = value;
      });
      watch(function () {
        return props.open;
      }, function (value) {
        setOpenState(value);
      });
      var onPickerChange = function onPickerChange(pickerValue) {
        if (state.pickerValue !== pickerValue) {
          var _picker$value;
          state.pickerValue = pickerValue;
          var pickerValueChangeProp = props.pickerValueChangeProp;
          if ((_picker$value = picker.value) !== null && _picker$value !== void 0 && _picker$value.props[pickerValueChangeProp]) {
            picker.value.props[pickerValueChangeProp](pickerValue);
          }
        }
      };
      var saveRef = function saveRef(picker) {
        picker.value = picker;
      };
      var setOpenState = function setOpenState(open) {
        state.open = open;
        if (!open) {
          state.pickerValue = null;
        }
      };
      var fireOpenChange = function fireOpenChange(open) {
        if (state.open !== open) {
          setOpenState(open);
          emit('open-change', open);
          emit('update:open', open);
        }
      };
      var onTriggerClick = function onTriggerClick(e) {
        var child = slots.default()[0];
        var childProps = child.props || {};
        if (childProps[props.triggerType]) {
          childProps[props.triggerType](e);
        }
        fireOpenChange(!state.open);
      };
      var onOk = function onOk() {
        emit('ok');
        fireOpenChange(false);
      };
      var getContent = function getContent() {
        if (slots.picker) {
          var localPicker = slots.picker()[0];
          var pickerValue = state.pickerValue;
          if (pickerValue === null) {
            pickerValue = props.value;
          }
          setProps(picker.value, _defineProperty(_defineProperty({}, props.pickerValueProp, pickerValue), props.pickerValueChangeProp, onPickerChange));

          // localPicker.ref = 'picker';
          return localPicker;
        }
        if (picker.value) {
          var _pickerValue = state.pickerValue;
          if (_pickerValue === null) {
            _pickerValue = props.value;
          }
          return cloneVNodes(picker.value, _defineProperty(_defineProperty(_defineProperty({}, props.pickerValueProp, _pickerValue), props.pickerValueChangeProp, onPickerChange), "ref", saveRef));
        } else {
          return props.picker;
        }
      };
      var onCancel = function onCancel() {
        fireOpenChange(false);
        emit('cancel');
      };
      var hide = function hide() {
        fireOpenChange(false);
        emit('hide');
      };
      return {
        getContent: getContent,
        onOk: onOk,
        hide: hide,
        onCancel: onCancel,
        state: state,
        onTriggerClick: onTriggerClick
      };
    },
    render: function render() {
      var _this = this;
      var props = this.$props;
      var children = unwrapFragment(this.$slots.default());
      if (!children) {
        return getModal(props, this.state.open, {
          getContent: this.getContent,
          onOk: this.onOk,
          hide: this.hide,
          onCancel: this.onCancel
        });
      }
      var disabled = this.$props.disabled;
      if (!disabled) {
        children.forEach(function (child) {
          setListeners(child, _defineProperty({}, _this.triggerType, _this.onTriggerClick));
        });
      }
      var modal = getModal(props, this.state.open, {
        getContent: this.getContent,
        onOk: this.onOk,
        hide: this.hide,
        onCancel: this.onCancel
      });
      return _createVNode(_Fragment, null, [children, modal]);
    }
  });
}